from wetb.hawc2.htc_file import HTCFile
import os
from os import path

pbs_template = '''### Standard Output
#PBS -N [jobname]
#PBS -o [pbsoutdir]/[jobname].out
### Standard Error
#PBS -e [pbsoutdir]/[jobname].err
#PBS -W umask=0003
### Maximum wallclock time format HOURS:MINUTES:SECONDS
#PBS -l walltime=[walltime]
#PBS -l nodes=1:ppn=1
### Queue name
#PBS -q workq

# ==============================================================================
# single PBS mode: one case per PBS job
# evaluates to true if LAUNCH_PBS_MODE is NOT set
if [ -z ${LAUNCH_PBS_MODE+x} ] ; then
  ### Create scratch directory and copy data to it
  cd $PBS_O_WORKDIR
  echo "current working dir (pwd):"
  pwd
  cp -R ./[modelzip] /scratch/$USER/$PBS_JOBID
fi
# ==============================================================================


# ==============================================================================
# single PBS mode: one case per PBS job
# evaluates to true if LAUNCH_PBS_MODE is NOT set
if [ -z ${LAUNCH_PBS_MODE+x} ] ; then
  echo
  echo 'Execute commands on scratch nodes'
  cd /scratch/$USER/$PBS_JOBID
  # create unique dir for each CPU
  mkdir "1"; cd "1"
  pwd
  /usr/bin/unzip ../[modelzip]
  mkdir -p [htcdir]
  mkdir -p [resdir]
  mkdir -p [logdir]
  mkdir -p [turbdir]
  cp -R $PBS_O_WORKDIR/[htcdir]/[jobname].htc ./[htcdir]
  cp -R $PBS_O_WORKDIR/[turbdir][turbfileroot]*.bin [turbdir]
  _HOSTNAME_=`hostname`
  if [[ ${_HOSTNAME_:0:1} == "j" ]] ; then
    WINEARCH=win64 WINEPREFIX=~/.wine winefix
  fi
# ==============================================================================

# ------------------------------------------------------------------------------
# find+xargs mode: 1 PBS job, multiple cases
else
  # with find+xargs we first browse to CPU folder
  cd "$CPU_NR"
fi
# ------------------------------------------------------------------------------

echo ""
# ==============================================================================
# single PBS mode: one case per PBS job
# evaluates to true if LAUNCH_PBS_MODE is NOT set
if [ -z ${LAUNCH_PBS_MODE+x} ] ; then
  echo "execute HAWC2, fork to background"
  time WINEARCH=win64 WINEPREFIX=~/.wine wine hawc2-latest ./[htcdir]/[jobname].htc &
  wait
# ==============================================================================

# ------------------------------------------------------------------------------
# find+xargs mode: 1 PBS job, multiple cases
else
  echo "execute HAWC2, do not fork and wait"
  (time WINEARCH=win64 WINEPREFIX=~/.wine numactl --physcpubind=$CPU_NR wine hawc2-latest ./[htcdir]/[jobname].htc) 2>&1 | tee [pbsoutdir]/[jobname].err.out
fi
# ------------------------------------------------------------------------------


### Epilogue
# ==============================================================================
# single PBS mode: one case per PBS job
# evaluates to true if LAUNCH_PBS_MODE is NOT set
if [ -z ${LAUNCH_PBS_MODE+x} ] ; then
  ### wait for jobs to finish
  wait
  echo ""
  echo "Copy back from scratch directory"
  mkdir -p $PBS_O_WORKDIR/[resdir]
  mkdir -p $PBS_O_WORKDIR/[logdir]
  mkdir -p $PBS_O_WORKDIR/animation/
  mkdir -p $PBS_O_WORKDIR/[turbdir]
  cp -R [resdir]. $PBS_O_WORKDIR/[resdir].
  cp -R [logdir]. $PBS_O_WORKDIR/[logdir].
  cp -R animation/. $PBS_O_WORKDIR/animation/.

  echo ""
  echo "COPY BACK TURB IF APPLICABLE"
  cd turb/
  for i in `ls *.bin`; do  if [ -e $PBS_O_WORKDIR/[turbdir]$i ]; then echo "$i exists no copyback"; else echo "$i copyback"; cp $i $PBS_O_WORKDIR/[turbdir]; fi; done
  cd /scratch/$USER/$PBS_JOBID/1/
  echo "END COPY BACK TURB"
  echo ""

  echo "COPYBACK [copyback_files]/[copyback_frename]"
  echo "END COPYBACK"
  echo ""
  echo ""
  echo "following files are on node/cpu 1 (find .):"
  find .
# ==============================================================================
# ------------------------------------------------------------------------------
# find+xargs mode: 1 PBS job, multiple cases
else
  cd /scratch/$USER/$PBS_JOBID/$CPU_NR/
  rsync -a --remove-source-files [resdir]. ../HAWC2SIM/[resdir].
  rsync -a --remove-source-files [logdir]. ../HAWC2SIM/[logdir].
  rsync -a --remove-source-files [pbsoutdir]/. ../HAWC2SIM/[pbsoutdir]/.
  rsync -a --remove-source-files animation/. ../HAWC2SIM/animation/.

  echo ""
  echo "COPY BACK TURB IF APPLICABLE"
  cd turb/
  for i in `ls *.bin`; do  if [ -e $PBS_O_WORKDIR/[turbdir]$i ]; then echo "$i exists no copyback"; else echo "$i copyback"; cp $i $PBS_O_WORKDIR/[turbdir]; fi; done
  cd /scratch/$USER/$PBS_JOBID/$CPU_NR/
  echo "END COPY BACK TURB"
  echo ""

  echo "COPYBACK [copyback_files]/[copyback_frename]"
  echo "END COPYBACK"
  echo ""
# ------------------------------------------------------------------------------
fi
exit
'''

def htc2pbs(htc_fn, walltime='00:40:00', zipfile=None):
    """
    Creates a PBS launch file (.p) based on a HAWC2 .htc file.
    - Assumes htc files are within a htc/[casename]/ directory relative to current directory.
    - Assumes there is a .zip file in the current directory which contains the turbine model.
      If there is none, the zip file is set to 'model.zip' by default
    - Will place a .p fine in pbs_in/[casename]/ directory relative to current directory.
    -

    Parameters
    ----------
    htc_fn : str
        The file name and path to the .htc file relative to current directory.
    walltime: str (default='00:40:00')
        A string indicating the walltime of the job of the form 'HH:MM:SS'
    zipfile: str (default=None)
        The filename of the zipfile containing the wind turbine model files and
        HAWC2 executable. if zipfile=None, searches the current directory for a
        zip file. If none is found, sets zipfile to 'model.zip'


    Returns
    -------
    str
        The filename and path to the output .p file

    Raises
    ------
    FileNotFoundError: If the file structure is not correct.
    """



    basename = path.relpath(path.dirname(htc_fn), 'htc')
    jobname = path.splitext(path.basename(htc_fn))[0]
    pbs_in_dir = path.join('pbs_in', basename)
    if basename == '.':
        raise FileNotFoundError('File structure is incorrect.')

    if not zipfile:
        try:
            zipfile = [x for x in os.listdir() if x.lower().endswith('.zip')][0]
        except:
            print('No .zip file found in current directory. Set model zip to \'model.zip\'')
            zipfile = 'model.zip'

    #   get the required parameters for the pbs file from the htc file
    htc = HTCFile(htc_fn) #modelpath='../..')
    p = {
        'walltime'      : walltime,
        'modelzip'      : zipfile,
        'jobname'       : jobname,
        'htcdir'        : 'htc/' + basename,
        'logdir'        :  path.dirname(htc.simulation.logfile.str_values())[2:] + '/',
        'resdir'        : path.dirname(htc.output.filename.str_values())[2:] + '/',
        'turbdir'       : path.dirname(htc.wind.mann.filename_u.str_values()) + '/',
        'turbfileroot'  : path.basename(htc.wind.mann.filename_u.str_values()).split('u.')[0],
        'pbsoutdir'     : 'pbs_out/' + basename
        }


    #Write pbs file based on template file and tags
    if not os.path.exists(pbs_in_dir):
        os.makedirs(pbs_in_dir)

    template = str(pbs_template)

    for key, value in p.items():
        template = template.replace('[' + key + ']', value)

    with open(os.path.join(pbs_in_dir, jobname + '.p'), 'w') as f:
        f.write(template)



# -*- coding: utf-8 -*-
"""
Created on Mon Mar  5 16:00:02 2012

@author: dave
"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()




# time and data should be 1 dimensional arrays
def array_1d(array):
    """
    Check if the given array has only one dimension. Following formats will
    return True:
        (x,), (x,1) and (1,x)
    """
    if not len(array.shape) == 1:
        # in case it has (samples,1) or (1,samples) as dimensions
        if len(array.shape) == 2:
            if (array.shape[0] == 1) or (array.shape[1] == 1):
                return True
            else:
                raise ValueError('only 1D arrays are accepted')
    else:
        return True

    return True



'''
Created on 29/10/2013

@author: mmpe
'''
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()
from wetb.hawc2 import Hawc2io
import numpy as np
import os
import sys
import unittest
from wetb.hawc2.ascii2bin.ascii2bin import ascii2bin, size_from_file

testfilepath = os.path.join(os.path.dirname(__file__), 'test_files/')  # test file path


class TextUI(object):
    def show_message(self, m):
        pass

    def exec_long_task(self, text, allow_cancel, task, *args, **kwargs):
        return task(*args, **kwargs)

class TestAscii2Bin(unittest.TestCase):


    def testAscii2bin(self):

        for f in ["Hawc2ascii_bin.sel", "Hawc2ascii_bin.dat"]:
            if os.path.exists(testfilepath + f):
                os.remove(testfilepath + f)
        ascii2bin(testfilepath + "Hawc2ascii.sel", ui=TextUI())

        ascii_file = Hawc2io.ReadHawc2(testfilepath + 'Hawc2ascii')
        bin_file = Hawc2io.ReadHawc2(testfilepath + "Hawc2ascii_bin")

        np.testing.assert_array_almost_equal(ascii_file.ReadAscii(), bin_file.ReadBinary(), 1)
        self.assertEqual(ascii_file.ChInfo, bin_file.ChInfo)

    def testAscii2bin_new_name(self):
        for f in ["Hawc2bin.sel", "Hawc2bin.dat"]:
            if os.path.exists(testfilepath + f):
                os.remove(testfilepath + f)
        ascii2bin(testfilepath + "Hawc2ascii.sel", testfilepath + "Hawc2bin.sel", ui=TextUI())

        ascii_file = Hawc2io.ReadHawc2(testfilepath + 'Hawc2ascii')
        bin_file = Hawc2io.ReadHawc2(testfilepath + "Hawc2bin")

        np.testing.assert_array_almost_equal(ascii_file.ReadAscii(), bin_file.ReadBinary(), 1)
        self.assertEqual(ascii_file.ChInfo, bin_file.ChInfo)


    def testSizeOfFile(self):
        self.assertEqual(size_from_file(testfilepath + "Hawc2ascii.sel"), (800, 28))



if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



'''
Created on 06/09/2013

@author: Mads M. Pedersen (mmpe@dtu.dk)
'''
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()
from wetb.hawc2.ascii2bin.ascii2bin import ascii2bin

if __name__=="__main__":
    ascii2bin(r"tests/hawc2ascii.sel", "temp_hawc2ascii.sel")




'''
Created on 16/07/2013

@author: mmpe
'''
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from future import standard_library
import sys
standard_library.install_aliases()

import unittest

import numpy as np
from wetb.fatigue_tools.fatigue import (eq_load, rainflow_astm,
                                        rainflow_windap, cycle_matrix)
from wetb.hawc2 import Hawc2io
import os

testfilepath = os.path.join(os.path.dirname(__file__), 'test_files/')  # test file path


class TestFatigueTools(unittest.TestCase):

    def test_leq_1hz(self):
        """Simple test of wetb.fatigue_tools.fatigue.eq_load using a sine
        signal.
        """
        amplitude = 1
        m = 1
        point_per_deg = 100

        for amplitude in [1, 2, 3]:
            peak2peak = amplitude * 2
            # sine signal with 10 periods (20 peaks)
            nr_periods = 10
            time = np.linspace(0, nr_periods * 2 * np.pi, point_per_deg * 180)
            neq = time[-1]
            # mean value of the signal shouldn't matter
            signal = amplitude * np.sin(time) + 5
            r_eq_1hz = eq_load(signal, no_bins=1, m=m, neq=neq)[0]
            r_eq_1hz_expected = ((2 * nr_periods * amplitude**m) / neq)**(1 / m)
            np.testing.assert_allclose(r_eq_1hz, r_eq_1hz_expected)

            # sine signal with 20 periods (40 peaks)
            nr_periods = 20
            time = np.linspace(0, nr_periods * 2 * np.pi, point_per_deg * 180)
            neq = time[-1]
            # mean value of the signal shouldn't matter
            signal = amplitude * np.sin(time) + 9
            r_eq_1hz2 = eq_load(signal, no_bins=1, m=m, neq=neq)[0]
            r_eq_1hz_expected2 = ((2 * nr_periods * amplitude**m) / neq)**(1 / m)
            np.testing.assert_allclose(r_eq_1hz2, r_eq_1hz_expected2)

            # 1hz equivalent should be independent of the length of the signal
            np.testing.assert_allclose(r_eq_1hz, r_eq_1hz2)

    def test_rainflow_combi(self):
        """Signal with two frequencies and amplitudes
        """

        amplitude = 1
        # peak2peak = amplitude * 2
        m = 1
        point_per_deg = 100

        nr_periods = 10
        time = np.linspace(0, nr_periods * 2 * np.pi, point_per_deg * 180)

        signal = (amplitude * np.sin(time)) + 5 + (amplitude * 0.2 * np.cos(5 * time))
        cycles, ampl_bin_mean, ampl_edges, mean_bin_mean, mean_edges = \
            cycle_matrix(signal, ampl_bins=10, mean_bins=5)

        cycles.sum()

    def test_astm1(self):

        signal = np.array([-2.0, 0.0, 1.0, 0.0, -3.0, 0.0, 5.0, 0.0, -1.0, 0.0, 3.0, 0.0, -4.0, 0.0, 4.0, 0.0, -2.0])

        ampl, mean = rainflow_astm(signal)
        np.testing.assert_array_equal(np.histogram2d(ampl, mean, [6, 4])[0], np.array([[0., 1., 0., 0.],
                                                                                       [1., 0., 0., 2.],
                                                                                       [0., 0., 0., 0.],
                                                                                       [0., 0., 0., 1.],
                                                                                       [0., 0., 0., 0.],
                                                                                       [0., 0., 1., 2.]]))

    def test_windap1(self):
        signal = np.array([-2.0, 0.0, 1.0, 0.0, -3.0, 0.0, 5.0, 0.0, -1.0, 0.0, 3.0, 0.0, -4.0, 0.0, 4.0, 0.0, -2.0])
        ampl, mean = rainflow_windap(signal, 18, 2)
        np.testing.assert_array_equal(np.histogram2d(ampl, mean, [6, 4])[0], np.array([[0., 0., 1., 0.],
                                                                                       [1., 0., 0., 2.],
                                                                                       [0., 0., 0., 0.],
                                                                                       [0., 0., 0., 1.],
                                                                                       [0., 0., 0., 0.],
                                                                                       [0., 0., 2., 1.]]))

    def test_windap2(self):
        data = Hawc2io.ReadHawc2(testfilepath + "test").ReadBinary([2]).flatten()
        np.testing.assert_allclose(eq_load(data, neq=61), np.array([[1.356, 1.758, 2.370, 2.784, 3.077, 3.296]]), 0.01)

    def test_astm2(self):
        data = Hawc2io.ReadHawc2(testfilepath + "test").ReadBinary([2]).flatten()
        np.testing.assert_allclose(eq_load(data, neq=61, rainflow_func=rainflow_astm),
                                   np.array([[1.356, 1.758, 2.370, 2.784, 3.077, 3.296]]), 0.01)


#     def test_windap3(self):
#         data = Hawc2io.ReadHawc2(testfilepath + "test").ReadBinary([2]).flatten()
#         from wetb.fatigue_tools.rainflowcounting import peak_trough
#         self.assertTrue(peak_trough.__file__.lower()[-4:] == ".pyd" or peak_trough.__file__.lower()[-3:] == ".so",
#                         "not compiled, %s, %s\n%s"%(sys.executable, peak_trough.__file__, os.listdir(os.path.dirname(peak_trough.__file__))))
#         np.testing.assert_array_equal(cycle_matrix(data, 4, 4, rainflow_func=rainflow_windap)[0], np.array([[  14., 65., 39., 24.],
#                                                                    [  0., 1., 4., 0.],
#                                                                    [  0., 0., 0., 0.],
#                                                                    [  0., 1., 2., 0.]]) / 2)

    def test_astm3(self):
        data = Hawc2io.ReadHawc2(testfilepath + "test").ReadBinary([2]).flatten()
        np.testing.assert_allclose(cycle_matrix(data, 4, 4, rainflow_func=rainflow_astm)[0], np.array([[24., 83., 53., 26.],
                                                                                                       [0., 1., 4., 0.],
                                                                                                       [0., 0., 0., 0.],
                                                                                                       [0., 1., 2., 0.]]) / 2, 0.001)

    def test_astm_weighted(self):
        data = Hawc2io.ReadHawc2(testfilepath + "test").ReadBinary([2]).flatten()
        np.testing.assert_allclose(cycle_matrix([(1, data), (1, data)], 4, 4, rainflow_func=rainflow_astm)[0], np.array([[24., 83., 53., 26.],
                                                                                                                         [0., 1.,
                                                                                                                             4., 0.],
                                                                                                                         [0., 0.,
                                                                                                                             0., 0.],
                                                                                                                         [0., 1., 2., 0.]]), 0.001)

    def test_astm_matlab_example(self):
        # example from https://se.mathworks.com/help/signal/ref/rainflow.html
        fs = 512

        X = np.array([-2, 1, -3, 5, -1, 3, -4, 4, -2])

        Y = -np.diff(X)[:, np.newaxis] / 2. * np.cos(np.pi *
                                                     np.arange(0, 1, 1 / fs))[np.newaxis] + ((X[:-1] + X[1:]) / 2)[:, np.newaxis]
        Y = np.r_[Y.flatten(), X[-1]]
        range_lst, mean_lst = (rainflow_astm(Y))
        np.testing.assert_array_equal(range_lst, [3, 4, 4, 4, 8, 9, 8, 6])
        np.testing.assert_array_equal(mean_lst, [-.5, -1, 1, 1, 1, .5, 0, 1])
        if 0:
            import matplotlib.pyplot as plt
            plt.plot(np.arange(0, len(X) - 1 + 1 / fs, 1 / fs), Y)
            plt.plot(np.arange(len(X)), X, 'o')
            plt.show()


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



'''
Created on 10/03/2014

@author: MMPE
'''
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from builtins import range
from builtins import str
from future import standard_library
import glob
import re
standard_library.install_aliases()

import os


DEBUG = False
def pexec(args, cwd=None):
    """
    usage: errorcode, stdout, stderr, cmd = pexec("MyProgram.exe arg1, arg2", r"c:\tmp\")

    """
    import subprocess
    if not isinstance(args, (list, tuple)):
        args = [args]
    args = [str(arg) for arg in args]
    for i in range(len(args)):
        if os.path.exists(args[i]):
            args[i] = str(args[i]).replace('/', os.path.sep).replace('\\', os.path.sep).replace('"', '')

    cmd = "%s" % '{} /c "{}"'.format (os.environ.get("COMSPEC", "cmd.exe"), subprocess.list2cmdline(args))
    if cwd and os.path.isfile(cwd):
        cwd = os.path.dirname(cwd)
    proc = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True, cwd=cwd)
    stdout, stderr = proc.communicate()
    errorcode = proc.returncode

    return errorcode, stdout.decode('cp1252'), stderr.decode('cp1252'), cmd


def process(args, cwd=None):
    import subprocess
    if not isinstance(args, (list, tuple)):
        args = [args]
    args = [str(arg) for arg in args]
    for i in range(len(args)):
        if os.path.exists(args[i]):
            args[i] = str(args[i]).replace('/', os.path.sep).replace('\\', os.path.sep).replace('"', '')

    cmd = "%s" % '{} /c "{}"'.format (os.environ.get("COMSPEC", "cmd.exe"), subprocess.list2cmdline(args))
    if cwd is not None and os.path.isfile(cwd):
        cwd = os.path.dirname(cwd)
    return subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False, cwd=cwd)

def exec_process(process):
    stdout, stderr = process.communicate()
    errorcode = process.returncode

    return errorcode, stdout.decode(), stderr.decode()




'''
Created on 29. mar. 2017

@author: mmpe
'''

import numpy as np
def differentiation(x, type='center', sample_frq=None, cutoff_frq=None):
    """Differentiate the signal
    
    Parameters
    ----------
    x : array_like
        The input signal
    type : {'right','center','left'}
        right: change from current to next observation\n
        center: average change from previous to next observation\n
        left: change from previous to current observation\n
    sample_frq : int, float or None, optional
        sample frequency of signal (only required if low pass filer is applied)
    cutoff_frq : int, float or None, optional
        Low pass filter cut off (frequencies higher than this frequency will be suppressed)
    Returns
    -------
    y : ndarray
        differentiated signal
    
    
    Examples
    --------
    >>> differentiation([1,2,1,0,1,1])
    """ 
 
            
    if cutoff_frq is not None:
        assert sample_frq is not None, "Argument sample_frq must be set to apply low pass filter"
        from wetb.signal.filters.frq_filters import low_pass
        x = low_pass(x, sample_frq, cutoff_frq)
    else:
        x = np.array(x) 
    if type=="left":
        dy = np.r_[np.nan, x[1:]-x[:-1]]
    elif type=="center": 
        dy = np.r_[x[1]-x[0], (x[2:]-x[:-2])/2, x[-1]-x[-2]]
    elif type=="right":
        dy = np.r_[x[1:]-x[:-1], np.nan]
    else:
        raise NotImplementedError()
    return dy


import cython
import numpy as np
cimport numpy as np
'''
Created on 29/05/2013

@author: Mads M. Pedersen (mmpe@dtu.dk)
'''


import cython
import numpy as np
cimport numpy as np


@cython.ccall
@cython.locals(alpha=cython.float, i=cython.int)
cpdef cy_low_pass_filter(np.ndarray[double,ndim=1] inp, double delta_t, double tau):
    cdef np.ndarray[double,ndim=1] output
    output = np.empty_like(inp, dtype=np.float)
    output[0] = inp[0]

    alpha = delta_t / (tau + delta_t)
    for i in range(1, inp.shape[0]):
        output[i] = output[i - 1] + alpha * (inp[i] - output[i - 1])  # Same as output[i] = alpha*inp[i]+(1-alpha)*output[i-1]

    return output

cpdef cy_dynamic_low_pass_filter(np.ndarray[double,ndim=1] inp, double delta_t, np.ndarray[double,ndim=1] tau, int method):
    cdef np.ndarray[double,ndim=1] output, alpha
    cdef int i

    output = np.empty_like(inp, dtype=np.float)
    output[0] = inp[0]

    if method == 1:
        alpha = delta_t / (tau + delta_t)
        for i in range(1, inp.shape[0]):
            output[i] = output[i - 1] + alpha[i] * (inp[i] - output[i - 1])  # Same as output[i] = alpha*inp[i]+(1-alpha)*output[i-1]
    elif method == 2:
        for i in range(1, inp.shape[0]):
            output[i] = (delta_t * (inp[i] + inp[i - 1] - output[i - 1]) + 2 * tau[i] * output[i - 1]) / (delta_t + 2 * tau[i])
    elif method == 3:
        for i in range(1, inp.shape[0]):
            output[i] = output[i - 1] * np.exp(-delta_t / tau[i]) + inp[i] * (1 - np.exp(-delta_t / tau[i]))
    return output


@cython.ccall
@cython.locals(alpha=cython.float, i=cython.int)
cpdef cy_high_pass_filter(np.ndarray[double,ndim=1] inp, double delta_t, double tau):
    cdef np.ndarray[double,ndim=1] output
    output = np.empty_like(inp, dtype=np.float)
    output[0] = inp[0]
    alpha = tau / (tau + delta_t)
    for i in range(1, inp.shape[0]):
        output[i] = alpha * (output[i - 1] + inp[i] - inp[i - 1])

    return output



'''
Created on 03/12/2015

@author: mmpe
'''
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()
import unittest
import numpy as np
import datetime
from wetb.gtsdf.unix_time import to_unix, from_unix


class TestUnixTime(unittest.TestCase):


    def test_to_unix(self):
        self.assertEqual(to_unix(datetime.datetime(2016, 2, 2, 13, 6, 25)), 1454418385)
        self.assertEqual(to_unix([datetime.datetime(2016, 2, 2, 13, 6, 25),datetime.datetime(2016, 2, 2, 13, 6, 26)]), [1454418385,1454418386])
        self.assertNotEqual(to_unix(datetime.datetime(2016, 2, 2, 13, 6, 26)), 1454418385)
        self.assertRaises(Exception, to_unix,1)


    def test_from_unix(self):
        self.assertEqual(from_unix(1454418385), datetime.datetime(2016, 2, 2, 13, 6, 25))
        self.assertNotEqual(from_unix(1454418385), datetime.datetime(2016, 2, 2, 13, 6, 26))
        
        self.assertEqual(from_unix(np.nan), datetime.datetime(1970,1,1,0,0))
        self.assertEqual(from_unix([1454418385,1454418386]), [datetime.datetime(2016, 2, 2, 13, 6, 25),datetime.datetime(2016, 2, 2, 13, 6, 26)])
        



if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



'''
Created on 24/04/2014

@author: MMPE
'''
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
from io import open
from builtins import range
from builtins import int
from future import standard_library
standard_library.install_aliases()

import numpy as np

class StFile(object):
    """Read HAWC2 St (beam element structural data) file

    Methods are autogenerated for:

    - r : curved length distance from main_body node 1 [m]
    - m : mass per unit length [kg/m]
    - x_cg : xc2-coordinate from C1/2 to mass center [m]
    - y_cg : yc2-coordinate from C1/2 to mass center [m]
    - ri_x : radius of gyration related to elastic center. Corresponds to rotation about principal bending xe axis [m]
    - ri_y : radius of gyration related to elastic center. Corresponds to rotation about principal bending ye axis [m]
    - xs : xc2-coordinate from C1/2 to shear center [m]. The shear center is the point where external forces only contributes to pure bending and no torsion.
    - ys : yc2-coordinate from C1/2 to shear center [m]. The shear center is the point where external forces only contributes to pure bending and no torsion.
    - E : modulus of elasticity [N/m2]
    - G : shear modulus of elasticity [N/m2]
    - Ix : area moment of inertia with respect to principal bending xe axis [m4]. This is the principal bending axis most parallel to the xc2 axis
    - Iy : area moment of inertia with respect to principal bending ye axis [m4]
    - K : torsional stiffness constant with respect to ze axis at the shear center [m4/rad]. For a circular section only this is identical to the polar moment of inertia.
    - kx : shear factor for force in principal bending xe direction [-]
    - ky : shear factor for force in principal bending ye direction [-]
    - A : cross sectional area [m2]
    - pitch : structural pitch about z_c2 axis. This is the angle between the xc2 -axis defined with the c2_def command and the main principal bending axis xe.
    - xe : xc2-coordinate from C1/2 to center of elasticity [m]. The elastic center is the point where radial force (in the z-direction) does not contribute to bending around the x or y directions.
    - ye : yc2-coordinate from C1/2 to center of elasticity [m]. The elastic center is the point where radial force (in the

    The autogenerated methods have the following structure

    def xxx(radius=None, mset=1, set=1):
        Parameters:
        -----------
        radius : int, float, array_like or None, optional
            Radius/radii of interest\n
            If int, float or array_like: values are interpolated to requested radius/radii
            If None (default): Values of all radii specified in st file returned
        mset : int, optional
            Main set number
        set : int, optional
            Sub set number


    Examples
    --------
    >>> stfile = StFile(r"tests/test_files/DTU_10MW_RWT_Blade_st.dat")
    >>> print (stfile.m()) # Interpolated mass at radius 36
    [ 1189.51054664  1191.64291781  1202.76694262  ... 15.42438683]
    >>> print (st.E(radius=36, mset=1, set=1))  # Elasticity interpolated to radius 36m
    8722924514.652649
    >>> print (st.E(radius=36, mset=1, set=2))  # Same for stiff blade set
    8.722924514652648e+17
    """
    def __init__(self, filename):
        with open (filename) as fid:
            txt = fid.read()
#         Some files starts with first set ("#1...") with out specifying number of sets
#         no_maindata_sets = int(txt.strip()[0]) 
#         assert no_maindata_sets == txt.count("#")
        self.main_data_sets = {}
        for mset in txt.split("#")[1:]:
            mset_nr = int(mset.strip().split()[0])
            set_data_dict = {}

            for set_txt in mset.split("$")[1:]:
                set_lines = set_txt.split("\n")
                set_nr, no_rows = map(int, set_lines[0].split()[:2])
                assert set_nr not in set_data_dict
                set_data_dict[set_nr] = np.array([set_lines[i].split() for i in range(1, no_rows + 1)], dtype=np.float)
            self.main_data_sets[mset_nr] = set_data_dict

        for i, name in enumerate("r m x_cg y_cg ri_x ri_y x_sh y_sh E G I_x I_y I_p k_x k_y A pitch x_e y_e".split()):
            setattr(self, name, lambda radius=None, mset=1, set=1, column=i: self._value(radius, column, mset, set))

    def _value(self, radius, column, mset_nr=1, set_nr=1):
        st_data = self.main_data_sets[mset_nr][set_nr]
        if radius is None:
            radius = self.radius_st(None, mset_nr, set_nr)
        return np.interp(radius, st_data[:, 0], st_data[:, column])

    def radius_st(self, radius=None, mset=1, set=1):
        r = self.main_data_sets[mset][set][:, 0]
        if radius is None:
            return r
        return r[np.argmin(np.abs(r - radius))]

    def to_str(self, mset=1, set=1):
        d = self.main_data_sets[mset][set]
        return "\n".join([("%12.5e "*d.shape[1]) % tuple(row) for row in d])


if __name__ == "__main__":
    import os
    st = StFile(os.path.dirname(__file__) + r"/tests/test_files/DTU_10MW_RWT_Blade_st.dat")
    print (st.m())
    print (st.E(radius=36, mset=1, set=1))  # Elastic blade
    print (st.E(radius=36, mset=1, set=2))  # stiff blade
    #print (st.radius())
    xyz = np.array([st.x_e(), st.y_e(), st.r()]).T[:40]
    n = 2
    xyz = np.array([st.x_e(None, 1, n), st.y_e(None, 1, n), st.r(None, 1, n)]).T[:40]
    #print (xyz)
    print (np.sqrt(np.sum((xyz[1:] - xyz[:-1]) ** 2, 1)).sum())
    print (xyz[-1, 2])
    print (np.sqrt(np.sum((xyz[1:] - xyz[:-1]) ** 2, 1)).sum() - xyz[-1, 2])
    print (st.x_e(67.8883), st.y_e(67.8883))
    #print (np.sqrt(np.sum(np.diff(xyz, 0) ** 2, 1)))
    print (st.pitch(67.8883 - 0.01687))
    print (st.pitch(23.2446))



    #print (st.)
    #print (st.)



"""
The 'General Time Series Data Format', gtsdf, is a binary hdf5 data format for storing time series data,\n
specified by \n
Mads M. Pedersen (mmpe@dtu.dk), DTU-Wind Energy, Aeroelastic design (AED)

Features:

-    Single file
-    Optional data type, e.g. 16bit integer (compact) or 64 bit floating point (high precision)
-    Precise time representation (including absolute times)
-    Additional data blocks can be appended continuously
-    Optional specification of name and description of dataset
-    Optional specification of name, unit and description of attributes
-    NaN support

This module contains three methods:

- load_
- save_
- append_block_

.. _load: gtsdf.html#gtsdf.load
.. _save: gtsdf.html#gtsdf.save
.. _append_block: gtsdf.html#gtsdf.append_block

"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()

d = None
d = dir()

from .gtsdf import save
from .gtsdf import load
from .gtsdf import append_block
from .gtsdf import load_pandas
from .gtsdf import add_statistic
from .gtsdf import load_statistic
from .gtsdf import compress2statistics

class Dataset(object):
    def __init__(self, filename):
        self.filename = filename
        self.time, self.data, self.info = load(filename)
    def __call__(self, id):
        if isinstance(id, str):
            if id=="Time":
                return self.time
            else:
                return self(self.info['attribute_names'].index(id) + 2)
        if id == 1:
            return self.time
        else:
            return self.data[:, id - 2]
        
    def attribute_names_str(self):
        return "\n".join(["1: Time"]+["%d: %s"%(i,n) for i, n in enumerate(self.info['attribute_names'],2)])
    
    def __getattribute__(self, name):
        try:
            return object.__getattribute__(self, name)
            
        except Exception as e:
            try:
                return self(name)
            except:
#                 for i, n in enumerate(self.info['attribute_names']):
#                     print (i,n)
                raise e
    
    def __contains__(self, name):
        return name in self.info['attribute_names']


__all__ = sorted([m for m in set(dir()) - set(d)])







"""
Contents
--------
- `fit <#wetb.wind.weibull.fit>`_: Fit a weibull distribution, in terms of the parameters k and A, to the provided wind speeds
- `pdf <#wetb.wind.weibull.pdf>`_: Create Weibull pdf function
- `random <#wetb.wind.weibull.random>`_: Create a list of n random Weibull distributed values
"""
import numpy as np
import math
gamma = math.gamma
def pdf(A, k):
    """Create Weibull pdf function

    Parameters
    ----------
    A : float
        Scale parameter
    k : float
        Shape parameter

    Returns
    -------
    pdf-function

    Examples
    --------
    >>> from wetb.wind import weibull
    >>> pdf_func = weibull.pdf(3,4)
    >>> pdf_func(5)
    0.131007116969
    >>> from pylab import arange, plot, show
    >>> wsp = arange(20)
    >>> plot(wsp, pdf_func(wsp))
    >>> show()

    """

    return lambda x: k * x ** (k - 1) / A ** k * np.exp(-(x / A) ** k)

def cdf(A,k):
    return lambda x: 1-np.exp(-(x/A)**k)

def random(A, k, n):
    """Create a list of n random Weibull distributed values

    Parameters
    ----------
    A : float
        Scale parameter
    k : float
        Shape parameter
    n : int
        Number of values

    Returns
    -------
    x : array_like, shape (n,)
        n random Weibull distributed values

    Examples
    --------
    >>> from wetb.wind import weibull
    >>> from pylab import hist, show
    >>> hist(weibull.random(4,2,1000), 20)
    >>> show()
    """
    return A * np.random.weibull(k, n)

def fit(wsp):
    """Fit a weibull distribution, in terms of the parameters k and A, to the provided wind speeds

    Parameters
    ----------
    wsp : array_like
        Wind speeds

    Returns
    -------
    A : float
        Scale parameter
    k : float
        Shape parameter

    Examples
    --------
    >>> from wetb.wind import weibull
    >>> A,k = weibull.fit(wsp_lst)
    """
    res_pr_ms = 2  # number of wind speed bins pr m/s

    pdf, x = np.histogram(wsp, bins=np.arange(0, np.ceil(np.nanmax(wsp)), 1 / res_pr_ms))
    x = (x[1:] + x[:-1]) / 2
    N = np.sum(~np.isnan(wsp))
    pdf = pdf / N * res_pr_ms

    m = lambda n : np.sum(pdf * x ** n / res_pr_ms)
    from scipy.optimize import newton
    func = lambda k : gamma(1 / k + 1) ** 2 / gamma(2 / k + 1) - m(1) ** 2 / m(2)
    k = newton(func, 1)
    A = m(1) / gamma(1 / k + 1)
    return A, k

if __name__ == "__main__":
    from wetb.wind import weibull
    from pylab import hist, show, plot
    hist(weibull.random(10, 2, 10000), 20, normed=True)
    wsp = np.arange(0, 20, .5)
    plot(wsp, weibull.pdf(10, 2)(wsp))
    show()



'''
Created on 15/01/2014

@author: MMPE
'''

import numpy as np
from wetb.utils.geometry import deg


def Ax(angle):
    cos = np.cos(angle)
    sin = np.sin(angle)
    return np.array([[1, 0, 0],
                   [0, cos, -sin],
                   [0, sin, cos]])

def Ay(angle):
    cos = np.cos(angle)
    sin = np.sin(angle)
    return np.array([[cos, 0, sin],
                   [0, 1, 0       ],
                   [-sin, 0, cos  ]])

def Az(angle):
    cos = np.cos(angle)
    sin = np.sin(angle)
    return np.array([[cos, -sin, 0],
                   [sin, cos, 0],
                   [0, 0, 1]])




def euler2A(euler_param):
    assert len(euler_param) == 4
    e = euler_param
    return np.array([[e[0] ** 2 + e[1] ** 2 - e[2] ** 2 - e[3] ** 2, 2 * (e[1] * e[2] + e[0] * e[3]) , 2 * (e[1] * e[3] - e[0] * e[2]) ],
                         [2 * (e[1] * e[2] - e[0] * e[3]), e[0] ** 2 - e[1] ** 2 + e[2] ** 2 - e[3] ** 2, 2 * (e[2] * e[3] + e[0] * e[1]) ],
                         [2 * (e[1] * e[3] + e[0] * e[2]), 2 * (e[2] * e[3] - e[0] * e[1]), e[0] ** 2 - e[1] ** 2 - e[2] ** 2 + e[3] ** 2]]).T

def A2euler(A):
    # method from http://www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToQuaternion/index.htm
    sqrt = np.sqrt
    (m00, m01, m02), (m10, m11, m12), (m20, m21, m22) = A
    tr = m00 + m11 + m22

    if (tr > 0):
        S = sqrt(tr + 1.0) * 2  # // S=4*qw
        qw = 0.25 * S
        qx = (m21 - m12) / S
        qy = (m02 - m20) / S
        qz = (m10 - m01) / S
    elif ((m00 > m11) and (m00 > m22)):
        S = sqrt(1.0 + m00 - m11 - m22) * 2  # // S=4*qx
        qw = (m21 - m12) / S
        qx = 0.25 * S
        qy = (m01 + m10) / S
        qz = (m02 + m20) / S
    elif (m11 > m22):
        S = sqrt(1.0 + m11 - m00 - m22) * 2  # // S=4*qy
        qw = (m02 - m20) / S
        qx = (m01 + m10) / S
        qy = 0.25 * S
        qz = (m12 + m21) / S
    else:
        S = sqrt(1.0 + m22 - m00 - m11) * 2  # // S=4*qz
        qw = (m10 - m01) / S
        qx = (m02 + m20) / S
        qy = (m12 + m21) / S
        qz = 0.25 * S
    return np.array([qw, qx, qy, qz])



#def A2xyz(A):
#    if abs(A[2, 0]) != 1:
#        y = -np.arcsin(A[2, 0])
#        x = np.arctan2(A[2, 1] / np.cos(y), A[2, 2] / np.cos(y))
#        z = np.arctan2(A[1, 0] / np.cos(y), A[0, 0] / np.cos(y))
#    else:
#        z = 0
#        if A[2, 0] == -1:
#            y = np.pi / 2
#            x = z + np.arctan(A[0, 1], A[0, 2])
#        else:
#            y = -np.pi / 2
#            x = -z + np.arctan(-A[0, 1], -A[0, 2])
#    return np.array([x, y, z])
#
#def zxz2euler(z1, x, z2):
#    return np.array([np.cos(.5 * (z1 + z2)) * np.cos(.5 * x),
#                     np.cos(.5 * (z1 - z2)) * np.sin(.5 * x),
#                     np.sin(.5 * (z1 - z2)) * np.sin(.5 * x),
#                     np.sin(.5 * (z1 + z2)) * np.cos(.5 * x)])
#
#def xyz2A(x, y, z):
#    return np.dot(Ax(x), np.dot(Ay(y), Az(z)))

#def euler2xyz(euler):
#    return A2xyz(euler2A(euler))

#def A2euler(A):
#    return xyz2euler(*A2xyz(A))

def euler2angle(euler):
    if euler[0] > 1:
        euler[0] = 1
    if euler[0] < -1:
        euler[0] = -1

    return np.arccos(euler[0]) * 2

def euler2gl(euler):
    return np.r_[deg(euler2angle(euler)), euler[1:]]



import numpy as np
def spline_fit(xp,yp):

        def akima(x, y):
            n = len(x)
            var = np.zeros((n + 3))
            z = np.zeros((n))
            co = np.zeros((n, 4))
            for i in range(n - 1):
                var[i + 2] = (y[i + 1] - y[i]) / (x[i + 1] - x[i])
            var[n + 1] = 2 * var[n] - var[n - 1]
            var[n + 2] = 2 * var[n + 1] - var[n]
            var[1] = 2 * var[2] - var[3]
            var[0] = 2 * var[1] - var[2]

            for i in range(n):
                wi1 = abs(var[i + 3] - var[i + 2])
                wi = abs(var[i + 1] - var[i])
                if (wi1 + wi) == 0:
                    z[i] = (var[i + 2] + var[i + 1]) / 2
                else:
                    z[i] = (wi1 * var[i + 1] + wi * var[i + 2]) / (wi1 + wi)

            for i in range(n - 1):
                dx = x[i + 1] - x[i]
                a = (z[i + 1] - z[i]) * dx
                b = y[i + 1] - y[i] - z[i] * dx
                co[i, 0] = y[i]
                co[i, 1] = z[i]
                co[i, 2] = (3 * var[i + 2] - 2 * z[i] - z[i + 1]) / dx
                co[i, 3] = (z[i] + z[i + 1] - 2 * var[i + 2]) / dx ** 2
            co[n - 1, 0] = y[n - 1]
            co[n - 1, 1] = z[n - 1]
            co[n - 1, 2] = 0
            co[n - 1, 3] = 0
            return co

        p_lst = [lambda x_, c=c, x0=x0: np.poly1d(c[::-1])(x_-x0) for c,x0 in zip(akima(xp,yp), xp)]
        
        def spline(x):
            y = np.empty_like(x)+np.nan
            segment = np.searchsorted(xp,x, 'right')-1
            for i in np.unique(segment):
                m = segment==i
                y[m] = p_lst[i](x[m])
            return y
#         def coef2spline(x, xp, co):
#             
#             print (np.searchsorted(xp,x)-1)
#             x, y = [], []
#             for i, c in enumerate(co.tolist()[:-1]):
#                 p = np.poly1d(c[::-1])
#                 z = np.linspace(0, s[i + 1] - s[i ], 10, endpoint=i >= co.shape[0] - 2)
#                 x.extend(s[i] + z)
#                 y.extend(p(z))
#             return y
#         
        return spline
        #x, y, z = [coef2spline(curve_z_nd, akima(curve_z_nd, self.c2def[:, i])) for i in range(3)]
        #return x, y, z
    
if __name__=="__main__":
    import matplotlib.pyplot as plt
    x = np.random.randint(0,100,10)
    t = np.arange(0,100,10)
    plt.plot(t,x,'.',label='points')
    
    t_ = np.arange(100)
    spline = spline_fit(t,x)
    print (np.abs(np.diff(np.diff(np.interp(t_, t,x)))).max())
    print (np.abs(np.diff(np.diff(spline(t_)))).max())
    plt.plot(t_, np.interp(t_, t,x))
    plt.plot(t_, spline(t_),label='spline')
    plt.show()



from wetb.hawc2.hawc2_input_writer import HAWC2InputWriter
from wetb.hawc2.tests import test_files
import os
from wetb.dlb.iec64100_1 import DTU_IEC64100_1_Ref_DLB

"""
TODO: delete wind ramp / replace wind section
TODO: set default turb_format = 0
"""


class HAWC2_IEC_DLC_Writer(HAWC2InputWriter):
    def __init__(self, base_htc_file, diameter,
                 time_start=100,  # Minimum 5s cf. IEC61400-1(2005), section 7.5
                 turbulence_defaults=(29.4, 3.7, 8192, 32)  # L, gamma, n_x, n_yz):
                 ):
        HAWC2InputWriter.__init__(self, base_htc_file, diameter=diameter,
                                  time_start=time_start,
                                  turbulence_defaults=turbulence_defaults)

    def set_V_hub(self, htc, V_hub, **_):
        htc.wind.wsp = V_hub
        htc.wind.wind_ramp_factor = 0, self.time_start, 8 / V_hub, 1

    def set_wdir(self, htc, wdir, **_):
        htc.wind.windfield_rotations = wdir, 0, 0

    def set_shear(self, htc, shear, **_):
        shear_format, shear_arg = shear['profile']
        i_shear_format = ['log', 'power'].index(shear_format.lower()) + 2
        htc.wind.shear_format = i_shear_format, shear_arg
        if shear['type'] == 'NWP':
            return
        elif shear['type'] == 'EWS':
            phi = {'++': 0, '+-': 90, '--': 180, '-+': 270}[shear['sign']]
            htc.wind.iec_gust = 'ews', shear['A'], phi, self.time_start, shear['T']
        else:
            raise NotImplementedError(shear['type'])

    def set_ti(self, htc, ti, **_):
        htc.wind.tint = ti

    def set_seed(self, htc, seed, **kwargs):
        #         if turbulence['type'] in ['NTM', 'ETM']:
        #             tint, seed = turbulence['ti'], turbulence['seed']
        #             htc.wind.tint = tint
        if seed is None:
            htc.wind.turb_format = 0
        elif isinstance(seed, int):
            L, Gamma, nx, nyz = self.turbulence_defaults

            htc.add_mann_turbulence(L, 1, Gamma, seed, no_grid_points=(nx, nyz, nyz),
                                    box_dimension=(kwargs['simulation_time'] * kwargs['V_hub'],
                                                   self.diameter, self.diameter))
    #         if turbulence['type'] == 'ECD':
    #             V_cg, theta_cg, T = [turbulence[k] for k in ['V_cg', 'theta_cg', 'T']]
    #             htc.wind.turb_format = 0
    #             htc.wind.iec_gust = 'ECD', V_cg, theta_cg, self.time_start, T
        else:
            raise NotImplementedError(seed)

    def set_Gust(self, htc, Gust, **kwargs):
        if str(Gust).lower() == 'nan':
            return
        elif Gust['type'] == 'ECD':
            V_cg, theta_cg, T = [Gust[k] for k in ['V_cg', 'theta_cg', 'T']]
            htc.wind.iec_gust = 'ECD', V_cg, theta_cg, self.time_start, T
        else:
            raise NotImplementedError(Gust)

    def set_Fault(self, htc, Fault, **kwargs):
        if str(Fault).lower() == 'nan':
            return
        elif Fault['type'] == 'GridLoss':
            T = Fault['T']
            self.set_gridloss_time(htc, self.time_start + T)
        else:
            raise NotImplementedError(Fault)

    def set_simulation_time(self, htc, simulation_time, **_):
        htc.set_time(self.time_start, simulation_time + self.time_start)

    def set_gridloss_time(self, htc, t):
        gen_servo = htc.dll.get_subsection_by_name('generator_servo', 'name')
        assert gen_servo.init.constant__7.comments == "Time for grid loss [s]"
        gen_servo.init.constant__7 = 7, t


if __name__ == '__main__':
    dlb = DTU_IEC64100_1_Ref_DLB(iec_wt_class='1A', Vin=4, Vout=26, Vr=10, D=180, z_hub=90)
    path = os.path.dirname(test_files.__file__) + '/simulation_setup/DTU10MWRef6.0/'
    writer = HAWC2_IEC_DLC_Writer(path + 'htc/DTU_10MW_RWT.htc', 180)
    p = writer.from_pandas(dlb, write_input_files=True)
    # print(p['DLC12'])



'''
Created on 30/04/2013

@author: Mads M. Pedersen (mmpe@dtu.dk)
'''

from ftplib import FTP
from zipfile import ZipFile
import io
import getpass


class FtpFile():

    def __init__(self, url, username, password):
        if url.lower().startswith('ftp://'):
            url = url[6:]
        self.url = url
        self.domain = url.split('/')[0]
        self.path = '/'.join(url.split('/')[1:])

        self.username = username
        if password is None:
            passwrod = getpass.getpass("Enter password for ftp server")
        self.password = password

    def load(self, open_file=None):
        ftp = FTP(self.domain)
        try:
            ftp.login(self.username, self.password)
            if open_file is None:
                open_file = self
            ftp.retrbinary('RETR /%s' % self.path, open_file.write)
        finally:
            ftp.close()
        open_file.seek(0)

    def write(self, obj):
        raise NotImplementedError

    def __str__(self):
        return "%s(url='%s', username='%s', password=None)" % (self.__class__.__name__, self.url, self.username)

    def name(self):
        return self.path.split('/')[-1]

    def filename(self):
        return self.path


class FtpTxtFile(io.StringIO, FtpFile):

    def __init__(self, url, username, password):
        FtpFile.__init__(self, url, username, password)
        io.StringIO.__init__(self)
        self.load()


    def write(self, s):
        if isinstance(s, bytes):
            s = s.decode('cp1251')
        io.StringIO.write(self, s)



class FtpZipFile(ZipFile, FtpFile):

    def __init__(self, url, username, password):
        FtpFile.__init__(self, url, username, password)
        byteIO = io.BytesIO()
        self.load(byteIO)
        ZipFile.__init__(self, byteIO)

    def FtpZipTxtFile(self, txt_filename):
        return FtpZipTxtFile(self.url, self.username, self.password, txt_filename)


class FtpZipTxtFile(io.StringIO, FtpZipFile):

    def __init__(self, url, username, password, filename):
        FtpZipFile.__init__(self, url, username, password)
        io.StringIO.__init__(self, buf='')
        fid = self.open(filename)
        self.buf = fid.read()
        if self.buf[-1] != "\n":
            self.buf += "\n"
        fid.close()
        self.filename = filename

    def __str__(self):
        return "%s(url='%s', username='%s', password=None, filename='%s')" % (self.__class__.__name__, self.url, self.username, self.filename)


class FtpBinaryFile(io.BytesIO, FtpFile):

    def __init__(self, url, username, password):
        FtpFile.__init__(self, url, username, password)
        self.load()

    def __str__(self):
        return FtpFile.__str__(self)



'''
Created on 24/06/2016

@author: MMPE
'''

import numpy as np
import unittest
from wetb.utils.geometry import rpm2rads
from _collections import deque
from tables.tests.test_index_backcompat import Indexes2_0TestCase
from wetb.signal.filters._differentiation import differentiation


def power_mean(power, trigger_indexes, I, rotor_speed, time, air_density=1.225, rotor_speed_mean_samples=1) :
    """Calculate the density normalized mean power, taking acceleration of the rotor into account

    Parameters
    ---------
    Power : array_like
        Power [W]
    trigger_indexes : array_like
        Trigger indexes
    I : float
        Rotor inerti [kg m^2]
    rotor_speed : array_like
        Rotor speed [rad/s]
    time : array_like
        time [s]
    air_density : int, float or array_like, optional
        Air density.
    rotor_speed_mean_samples : int
        To reduce the effect of noise, the mean of a number of rotor speed samples can be used

    Returns
    -------
    mean power including power used to (de)accelerate rotor

    Examples:
    ---------
    turbine_power_mean = lambda power, triggers : power_mean(power, triggers, I=2.5E7, rot_speed, time, rho)
    trigger_indexes = time_trigger(time,30)
    wsp_mean, power_mean = subset_mean([wsp, power],trigger_indexes,mean_func={1:turbine_power_mean})
    """
    if rotor_speed_mean_samples == 1:
        rs1 = rotor_speed[trigger_indexes[:-1]]
        rs2 = rotor_speed[trigger_indexes[1:] - 1]
    else:
        rs = np.array([rotor_speed[max(i - rotor_speed_mean_samples, 0):i - 1 + rotor_speed_mean_samples].mean() for i in trigger_indexes])
        rs1 = rs[:-1]
        rs2 = rs[1:]


    power = np.array([np.nanmean(power[i1:i2], 0) for i1, i2 in zip(trigger_indexes[:-1].tolist(), trigger_indexes[1:].tolist())])
    if isinstance(air_density, (int, float)):
        if air_density != 1.225:
            power = power / air_density * 1.225
    else:
        air_density = np.array([np.nanmean(air_density[i1:i2], 0) for i1, i2 in zip(trigger_indexes[:-1].tolist(), trigger_indexes[1:].tolist())])
        power = power / air_density * 1.225
    return power + 1 / 2 * I * (rs2 ** 2 - rs1 ** 2) / (time[trigger_indexes[1:] - 1] - time[trigger_indexes[:-1]])

def power_mean_func_kW(I, rotor_speed, time, air_density=1.225, rotor_speed_mean_samples=1) :
    """Return a power mean function [kW] used to Calculate the density normalized mean power, taking acceleration of the rotor into account

    Parameters
    ---------
    I : float
        Rotor inerti [kg m^2]
    rotor_speed : array_like
        Rotor speed [rad/s]
    time : array_like
        time [s]
    air_density : int, float or array_like, optional
        Air density.
    rotor_speed_mean_samples : int
        To reduce the effect of noise, the mean of a number of rotor speed samples can be used

    Returns
    -------
    mean power function

    Examples:
    ---------
    turbine_power_mean = power_mean_func_kW(power, triggers, I=2.5E7, rot_speed, time, rho)
    trigger_indexes = time_trigger(time,30)
    wsp_mean, power_mean = subset_mean([wsp, power],trigger_indexes,mean_func={1:turbine_power_mean})
    """
    def mean_power(power, trigger_indexes):
        return power_mean(power * 1000, trigger_indexes, I, rotor_speed, time , air_density, rotor_speed_mean_samples) / 1000
    return mean_power


def subset_mean(data, trigger_indexes, mean_func={}):
    if isinstance(data, list):
        data = np.array(data).T
    if len(data.shape)==1:
        no_sensors = 1
    else:
        no_sensors = data.shape[1]
    if isinstance(trigger_indexes[0], tuple):
        triggers = np.array(trigger_indexes)
        steps = np.diff(triggers[:, 0])
        lengths = np.diff(triggers)[:, 0]
        if np.all(steps == steps[0]) and np.all(lengths == lengths[0]):
            subset_mean = np.mean(np.r_[data.reshape(data.shape[0],no_sensors),np.empty((steps[0],no_sensors))+np.nan][triggers[0][0]:triggers.shape[0] * steps[0] + triggers[0][0]].reshape(triggers.shape[0], steps[0], no_sensors)[:, :lengths[0]], 1)
        else:
            subset_mean = np.array([np.mean(data[i1:i2], 0) for i1, i2 in trigger_indexes])
        for index, func in mean_func.items():
            att = data[:, index]
            subset_mean[:, index] = func(att, trigger_indexes)
    else:
        steps = np.diff(trigger_indexes)

        if np.all(steps == steps[0]):
            #equal distance
            subset_mean = np.mean(data[trigger_indexes[0]:trigger_indexes[-1]].reshape([ len(trigger_indexes) - 1, steps[0], data.shape[1]]), 1)
        else:
            subset_mean = np.array([np.mean(data[i1:i2], 0) for i1, i2 in zip(trigger_indexes[:-1].tolist(), trigger_indexes[1:].tolist())])
        for index, func in mean_func.items():
            att = data[:, index]
            subset_mean[:, index] = func(att, trigger_indexes)
    if len(data.shape)==1 and len(subset_mean.shape)==2:
        return subset_mean[:,0]
    else:
        return subset_mean


def cycle_trigger(values, trigger_value=None, step=1, ascending=True, tolerance=0):
    values = np.array(values)
    if trigger_value is None:
        r = values.max() - values.min()
        values = (values[:] - r / 2) % r
        trigger_value = r / 2
    if ascending:
        return np.where((values[1:] > trigger_value + tolerance) & (values[:-1] <= trigger_value - tolerance))[0][::step]
    else:
        return np.where((values[1:] < trigger_value - tolerance) & (values[:-1] >= trigger_value + tolerance))[0][::step]

def revolution_trigger(rotor_position, sample_frq, rotor_speed, max_rev_diff=1, plt=None):
    """Returns one index per revolution (minimum rotor position)
    
    Parameters
    ----------
    rotor_position : array_like
        Rotor position [deg] (0-360)
    sample_frq : int or float
        Sample frequency [Hz]
    rotor_speed : array_like
        Rotor speed [RPM]
        
    Returns
    -------
    nd_array : Array of indexes
    """
    if isinstance(rotor_speed, (float, int)):
        rotor_speed = np.ones_like(rotor_position)*rotor_speed
    deg_per_sample = rotor_speed*360/60/sample_frq
    thresshold = deg_per_sample.max()*3
    drp = (np.diff(rotor_position)+thresshold)%360-thresshold
    rp = rotor_position

    

    
    rp = np.array(rotor_position).copy()%360
    #filter degree increase > thresshold
    rp[np.r_[False, np.diff(rp)>thresshold]] = 180
    
    upper_indexes = np.where((rp[:-1]>(360-thresshold))&(rp[1:]<(360-thresshold)))[0]
    lower_indexes = np.where((rp[:-1]>thresshold)&(rp[1:]<thresshold))[0] +1 
    
    if plt:
        plt.plot(rp)
        plt.plot(lower_indexes, rp[lower_indexes],'.')
        plt.plot(upper_indexes, rp[upper_indexes],'.')
        

    # Best lower is the first lower after upper
    best_lower = lower_indexes[np.searchsorted(lower_indexes, upper_indexes)]
    upper2lower = best_lower - upper_indexes
    trigger_indexes = best_lower[upper2lower<upper2lower.mean()*2].tolist()

    if len(trigger_indexes)>1:
        
    
        
        rpm_rs = np.array([rev.mean() for rev in np.split(rotor_speed, trigger_indexes)[1:-1]])
        rpm_i = 1/np.diff(trigger_indexes)*60*sample_frq
        spr_rs = np.array([rev.mean() for rev in np.split(1/rotor_speed*60*sample_frq, trigger_indexes)[1:-1]])
        spr_i = np.diff(trigger_indexes)
        

        while np.any(spr_rs>spr_i*1.9):
            i = np.where(spr_rs>spr_i*1.9)[0][0]
            if np.abs(spr_i[i-1] + spr_i[i] - spr_rs[i])< np.abs(spr_i[i] + spr_i[i+1] - spr_rs[i]):
                del trigger_indexes[i]
            else:
                del trigger_indexes[i+1]
            spr_i = np.diff(trigger_indexes)
            spr_rs = np.array([rev.mean() for rev in np.split(1/rotor_speed*60*sample_frq, trigger_indexes)[1:-1]])
        
            
        
        # if a revolution is too long add trigger
        if np.any(rpm_rs>rpm_i*2.1):
            # >1 missing triggers
            raise NotImplementedError
        trigger_indexes.extend([np.mean(trigger_indexes[i:i+2]) for i in np.where(rpm_rs>rpm_i*1.9)[0]])
        trigger_indexes = np.sort(trigger_indexes).astype(np.int)
    
    
        i1,i2 = trigger_indexes[0], trigger_indexes[-1]
        nround_rotor_speed = np.nansum(rotor_speed[i1:i2]/60/sample_frq)
        #mod = [v for v in [5,10,30,60,90] if v>thresshold][0]    
        nround_rotor_position = len(trigger_indexes)-1 #np.nansum(np.diff(rotor_position[i1:i2])%mod)/360
        if max_rev_diff is not None:
            diff_pct = abs(nround_rotor_position-nround_rotor_speed)/nround_rotor_position*100
            assert diff_pct<max_rev_diff, "No of revolution mismatch: rotor_position (%d), rotor_speed (%.1f), diff %.1f%%"%(nround_rotor_position, nround_rotor_speed, diff_pct)
         
    
    return trigger_indexes
    

def revolution_trigger_old(values, rpm_dt=None, dmin=5, dmax=10, ):
    """Return indexes where values are > max(values)-dmin and decreases more than dmax
    If RPM and time step is provided, triggers steps < time of 1rpm is removed   
    
    Parameters
    ---------
    values : array_like
        Position signal (e.g. rotor position)
    rpm_dt : tuple(array_like, float), optional
        - rpm : RPM signal
        - dt : time step between samples
    dmin : int or float, optional
        Minimum normal position increase between samples
    dmax : float or int, optional
        Maximum normal position increase between samples 
            
    
    Returns
    -------
    trigger indexes 
        [i1,i2,...,in] if rpm_dt is not provided
        [(start1,stop1),(start2,stop2),...,(startn, stopn)] if rpm_dt is provided
    """
    
    values = np.array(values)
    indexes = np.where((np.diff(values)<-dmax)&(values[:-1]>values.max()-dmax))[0]
    
    if rpm_dt is None:
        return indexes
    else:
        index_pairs = []
        rpm, dt = rpm_dt
        d_deg = rpm *360/60*dt
        cum_d_deg = np.cumsum(d_deg)
        lbound, ubound = values.max()-dmax, values.max()+dmax
        index_pairs = [(i1,i2) for i1,i2, deg in zip(indexes[:-1], indexes[1:], cum_d_deg[indexes[1:]-1]-cum_d_deg[indexes[:-1]]) 
                       if deg > lbound and deg<ubound]
        return index_pairs
    
    
def time_trigger(time, step, start=None, stop=None):
    if start is None:
        start = time[0]
    decimals = int(np.ceil(np.log10(1 / np.nanmin(np.diff(time)))))
    time = np.round(time - start, decimals)


    steps = np.round(np.diff(time), decimals)
    if np.sum(steps == steps[0])/len(time)>.99: #np.all(steps == steps[0]):
        # equal time step
        time = np.r_[time, time[-1] + max(set(steps), key=list(steps).count)]
    if stop is None:
        stop = time[~np.isnan(time)][-1]
    else:
        stop -= start
    epsilon = 10 ** -(decimals + 2)
    return np.where(((time % step < epsilon) | (time % step > step - epsilon)) & (time >= 0) & (time <= stop))[0]


def non_nan_index_trigger(sensor, step):
    trigger = []
    i = 0
    nan_indexes = deque(np.where(np.isnan(sensor))[0].tolist() + [len(sensor)])
    while i + step <= sensor.shape[0]:
        if i+step<=nan_indexes[0]:
            trigger.append((i,i+step))
            i+=step
        else:
            i = nan_indexes.popleft()+1
    return trigger




'''
Created on 12/09/2013

@author: mmpe
'''
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from builtins import super
from builtins import range
from future import standard_library
standard_library.install_aliases()

import h5py
import numpy as np
from wetb import gtsdf

import unittest
import os

tmp_path = os.path.dirname(__file__) + "/tmp/"
tfp = os.path.dirname(os.path.abspath(__file__)) + "/test_files/"

class Test_gsdf(unittest.TestCase):
    def setUp(self):
        unittest.TestCase.setUp(self)
        if not os.path.isdir(tmp_path):
            os.makedirs(tmp_path)

    @classmethod
    def tearDownClass(cls):
        super(Test_gsdf, cls).tearDownClass()
        #shutil.rmtree(tmp_path)

    def test_minimum_requirements (self):
        fn = tmp_path + "minimum.hdf5"
        f = h5py.File(fn, "w")
        #no type
        self.assertRaises(ValueError, gtsdf.load, fn)
        f.attrs["type"] = "General time series data format"

        #no no_blocks
        self.assertRaises(ValueError, gtsdf.load, fn)
        f.attrs["no_blocks"] = 0

        #no block0000
        self.assertRaises(ValueError, gtsdf.load, fn)
        b = f.create_group("block0000")

        #no data
        self.assertRaises(ValueError, gtsdf.load, fn)
        b.create_dataset("data", data=np.empty((0, 0)))
        f.close()
        gtsdf.load(fn)


    def test_save_no_hdf5_ext(self):
        fn = tmp_path + "no_hdf5_ext"
        gtsdf.save(fn, np.arange(12).reshape(4, 3))
        _, _, info = gtsdf.load(fn + ".hdf5")
        self.assertEqual(info['name'], 'no_hdf5_ext')

    def test_load_filename(self):
        fn = tmp_path + "filename.hdf5"
        gtsdf.save(fn, np.arange(12).reshape(4, 3))
        _, _, info = gtsdf.load(fn)
        self.assertEqual(info['name'], 'filename')


    def test_load_fileobject(self):
        fn = tmp_path + "fileobject.hdf5"
        gtsdf.save(fn, np.arange(12).reshape(4, 3))
        _, _, info = gtsdf.load(fn)
        self.assertEqual(info['name'], 'fileobject')

    def test_save_wrong_no_attr_info(self):
        fn = tmp_path + "wrong_no_attr_info.hdf5"
        self.assertRaises(AssertionError, gtsdf.save, fn, np.arange(12).reshape(4, 3), attribute_names=['Att1'])
        self.assertRaises(AssertionError, gtsdf.save, fn, np.arange(12).reshape(4, 3), attribute_units=['s'])
        self.assertRaises(AssertionError, gtsdf.save, fn, np.arange(12).reshape(4, 3), attribute_descriptions=['desc'])

    def test_info(self):
        fn = tmp_path + "info.hdf5"
        gtsdf.save(fn, np.arange(12).reshape(6, 2),
                   name='datasetname',
                   description='datasetdescription',
                   attribute_names=['att1', 'att2'],
                   attribute_units=['s', 'm/s'],
                   attribute_descriptions=['d1', 'd2'])
        _, _, info = gtsdf.load(fn)
        self.assertEqual(info['name'], "datasetname")
        self.assertEqual(info['type'], "General time series data format")
        self.assertEqual(info['description'], "datasetdescription")
        self.assertEqual(list(info['attribute_names']), ['att1', 'att2'])
        self.assertEqual(list(info['attribute_units']), ['s', 'm/s'])
        self.assertEqual(list(info['attribute_descriptions']), ['d1', 'd2'])

    def test_no_time(self):
        fn = tmp_path + 'time.hdf5'
        gtsdf.save(fn, np.arange(12).reshape(6, 2))
        time, _, _ = gtsdf.load(fn)
        np.testing.assert_array_equal(time, np.arange(6))

    def test_int_time(self):
        fn = tmp_path + 'time.hdf5'
        gtsdf.save(fn, np.arange(12).reshape(6, 2), time=range(4, 10))
        time, _, _ = gtsdf.load(fn)
        np.testing.assert_array_equal(time, range(4, 10))


    def test_time_offset(self):
        fn = tmp_path + 'time.hdf5'
        gtsdf.save(fn, np.arange(12).reshape(6, 2), time=range(6), time_start=4)
        time, _, _ = gtsdf.load(fn)
        np.testing.assert_array_equal(time, range(4, 10))


    def test_time_gain_offset(self):
        fn = tmp_path + 'time.hdf5'
        gtsdf.save(fn, np.arange(12).reshape(6, 2), time=range(6), time_step=1 / 4, time_start=4)
        time, _, _ = gtsdf.load(fn)
        np.testing.assert_array_equal(time, np.arange(4, 5.5, .25))

    def test_float_time(self):
        fn = tmp_path + 'time.hdf5'
        gtsdf.save(fn, np.arange(12).reshape(6, 2), time=np.arange(4, 5.5, .25))
        time, _, _ = gtsdf.load(fn)
        np.testing.assert_array_equal(time, np.arange(4, 5.5, .25))

    def test_data(self):
        fn = tmp_path + 'data.hdf5'
        d = np.arange(12).reshape(6, 2)
        gtsdf.save(fn, d)
        f = h5py.File(fn)
        self.assertEqual(f['block0000']['data'].dtype, np.uint16)
        f.close()
        _, data, _ = gtsdf.load(fn)
        np.testing.assert_array_almost_equal(data, np.arange(12).reshape(6, 2), 4)

    def test_data_float(self):
        fn = tmp_path + 'time.hdf5'
        d = np.arange(12).reshape(6, 2)
        gtsdf.save(fn, d, dtype=np.float32)
        f = h5py.File(fn)
        self.assertEqual(f['block0000']['data'].dtype, np.float32)
        f.close()
        _, data, _ = gtsdf.load(fn)
        np.testing.assert_array_equal(data, np.arange(12).reshape(6, 2))


    def test_all(self):
        fn = tmp_path + "all.hdf5"
        gtsdf.save(fn, np.arange(12).reshape(6, 2),
                   name='datasetname',
                   time=range(6), time_step=1 / 4, time_start=4,
                   description='datasetdescription',
                   attribute_names=['att1', 'att2'],
                   attribute_units=['s', 'm/s'],
                   attribute_descriptions=['d1', 'd2'])
        time, data, info = gtsdf.load(fn)
        self.assertEqual(info['name'], "datasetname")
        self.assertEqual(info['type'], "General time series data format")
        self.assertEqual(info['description'], "datasetdescription")
        self.assertEqual(list(info['attribute_names']), ['att1', 'att2'])
        self.assertEqual(list(info['attribute_units']), ['s', 'm/s'])
        self.assertEqual(list(info['attribute_descriptions']), ['d1', 'd2'])
        np.testing.assert_array_equal(time, np.arange(4, 5.5, .25))
        np.testing.assert_array_almost_equal(data, np.arange(12).reshape(6, 2), 4)

    def test_append(self):
        fn = tmp_path + 'append.hdf5'
        d = np.arange(48, dtype=np.float32).reshape(24, 2)
        d[2, 0] = np.nan
        gtsdf.save(fn, d)
        _, data, _ = gtsdf.load(fn)
        np.testing.assert_array_almost_equal(data, d, 3)
        gtsdf.append_block(fn, d)
        _, data, _ = gtsdf.load(fn)
        self.assertEqual(data.shape, (48, 2))
        np.testing.assert_array_almost_equal(data, np.append(d, d, 0), 3)
        f = h5py.File(fn)
        self.assertIn('gains', f['block0001'])
        f.close()

    def test_append_small_block(self):
        fn = tmp_path + 'append_small_block.hdf5'
        d = np.arange(12, dtype=np.float32).reshape(2, 6)
        gtsdf.save(fn, d)
        gtsdf.append_block(fn, d + 12)
        f = h5py.File(fn)
        self.assertNotIn('gains', f['block0001'])
        f.close()


    def test_nan_float(self):
        fn = tmp_path + 'nan.hdf5'
        d = np.arange(12, dtype=np.float32).reshape(6, 2)
        d[2, 0] = np.nan
        gtsdf.save(fn, d)
        _, data, _ = gtsdf.load(fn)
        np.testing.assert_array_almost_equal(data, d, 4)



    def test_outlier(self):
        fn = tmp_path + 'outlier.hdf5'
        d = np.arange(12, dtype=np.float32).reshape(6, 2)
        d[2, 0] = 10 ** 4
        d[3, 1] = 10 ** 4
        self.assertRaises(Warning, gtsdf.save, fn, d)
        _, data, _ = gtsdf.load(fn)

    def test_inf(self):
        fn = tmp_path + 'outlier.hdf5'
        d = np.arange(12, dtype=np.float32).reshape(6, 2)
        d[2, 0] = np.inf
        d[3, 1] = 10 ** 3
        self.assertRaises(ValueError, gtsdf.save, fn, d)

    def test_loadpandas(self):
        fn = tmp_path + "all.hdf5"
        gtsdf.save(fn, np.arange(12).reshape(6, 2),
                   name='datasetname',
                   time=range(6), time_step=1 / 4, time_start=4,
                   description='datasetdescription',
                   attribute_names=['att1', 'att2'],
                   attribute_units=['s', 'm/s'],
                   attribute_descriptions=['d1', 'd2'])
        df = gtsdf.load_pandas(fn)

    def test_loadtesthdf5(self):
        time, data, info = gtsdf.load(tfp + 'test.hdf5')
        self.assertEqual(time[1], 0.05)
        self.assertEqual(data[1, 1], 11.986652374267578)
        self.assertEqual(info['attribute_names'][1], "WSP gl. coo.,Vy")
    
    def test_loadhdf5File(self):
        f = h5py.File(tfp + 'test.hdf5')
        time, data, info = gtsdf.load(f)
        
        self.assertEqual(time[1], 0.05)
        self.assertEqual(data[1, 1], 11.986652374267578)
        self.assertEqual(info['attribute_names'][1], "WSP gl. coo.,Vy")
        
        
    def test_gtsdf_dataset(self):
        ds = gtsdf.Dataset(tfp+'test.hdf5')
        self.assertEqual(ds.data.shape, (2440,49))
        self.assertEqual(ds('Time')[1], 0.05)
        self.assertEqual(ds.Time[1], 0.05)
        self.assertRaisesRegex(AttributeError, "'Dataset' object has no attribute 'Time1'", lambda : ds.Time1)
        self.assertEqual(ds(2)[1], 12.04148006439209)
        n = ds.info['attribute_names'][2]
        self.assertEqual(n, "WSP gl. coo.,Vy")
        self.assertEqual(ds(n)[1], 12.04148006439209)
        
        
        






if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



'''
Created on 04/12/2015

@author: mmpe
'''
import os
import io
from wetb.utils.cluster_tools.pbsfile import PBSFile
from wetb.utils.cluster_tools.os_path import relpath

NOT_SUBMITTED = "Job not submitted"
PENDING = "Pending"
RUNNING = "Running"
DONE = "Done"


def pjoin(*args):
    return os.path.join(*args).replace('\\', '/')


class SSHPBSJob(object):
    _status = NOT_SUBMITTED
    nodeid = None
    jobid = None

    def __init__(self, sshClient):
        self.ssh = sshClient

    def submit(self, pbsfile, cwd=None, pbs_out_file=None):
        self.cwd = cwd
        self.nodeid = None
        if isinstance(pbsfile, PBSFile):
            f = io.StringIO(str(pbsfile))
            f.seek(0)
            pbs_filename = pjoin(cwd, pbsfile.filename)
            self.ssh.upload(f, pbs_filename)
            self.pbs_out_file = pjoin(cwd, pbsfile.stdout_filename)
            cwd = pbsfile.workdir
            pbsfile = pbsfile.filename
        else:
            self.pbs_out_file = os.path.relpath(cwd + pbs_out_file).replace("\\", "/")
        cmds = ['rm -f %s' % self.pbs_out_file]
        if cwd != "":
            cmds.append("cd %s" % cwd)
        cmds.append("qsub %s" % pbsfile)
        _, out, _ = self.ssh.execute(";".join(cmds))
        self.jobid = out.split(".")[0]
        self._status = PENDING

    @property
    def status(self):
        if self._status in [NOT_SUBMITTED, DONE]:
            return self._status
        with self.ssh:
            if self.is_executing():
                self._status = RUNNING
            elif self.ssh.file_exists(self.pbs_out_file):
                self._status = DONE
                self.jobid = None
        return self._status

    def get_nodeid(self):
        try:
            _, out, _ = self.ssh.execute("qstat -f %s | grep exec_host" % self.jobid)
            return out.strip().replace("exec_host = ", "").split(".")[0]
        except Warning as e:
            if 'qstat: Unknown Job Id' in str(e):
                return None
            #raise e

    def stop(self):
        if self.jobid:
            try:
                self.ssh.execute("qdel %s" % self.jobid)
            except Warning as e:
                if 'qdel: Unknown Job Id' in str(e):
                    return
                raise e

    def is_executing(self):
        try:
            self.ssh.execute("qstat %s" % self.jobid)
            return True
        except Warning as e:
            if 'qstat: Unknown Job Id' in str(e):
                return False
            raise e



'''
Created on 22. mar. 2017

@author: mmpe
'''
import numpy as np
def linear_fit(x,y):
    from scipy.stats import linregress
    slope, intercept, r_value, p_value, std_err = linregress(x,y)
    return np.array([x.min(), x.max()]), lambda x : np.array(x)*slope+intercept , (slope, intercept)


from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from builtins import zip
from future import standard_library
standard_library.install_aliases()
from datetime import datetime, date
import numpy as np
timestamp0 = datetime.utcfromtimestamp(0)


def to_unix(dateTime):
    try:
        return (dateTime - timestamp0).total_seconds()
    except:
        if hasattr(dateTime, "__len__"):
            return [(dt - timestamp0).total_seconds() for dt in dateTime]
        raise

# def from_unix_old(sec):
#     if np.isnan(sec):
#         return datetime.utcfromtimestamp(0)
#     return datetime.utcfromtimestamp(sec)


day_dict = {}


def from_unix(sec):
    global day_dict
    if isinstance(sec, (float, int)):
        if np.isnan(sec):
            return datetime.utcfromtimestamp(0)
        return datetime.utcfromtimestamp(sec)
    else:
        sec = np.array(sec).astype(np.float)
        ms = np.atleast_1d((sec * 1000000 % 1000000).astype(np.int))
        sec = sec.astype(np.int)
        S = np.atleast_1d(sec % 60)
        M = np.atleast_1d(sec % 3600 // 60)
        H = np.atleast_1d(sec % 86400 // 3600)
        d = np.atleast_1d(sec // 86400)
        for du in np.unique(d):
            if du not in day_dict:
                day_dict[du] = date.fromordinal(719163 + du).timetuple()[:3]
        y, m, d = zip(*[day_dict[d_] for d_ in d])
        return ([datetime(*ymdhmsu) for ymdhmsu in zip(y, m, d, H.tolist(), M.tolist(), S.tolist(), ms.tolist())])



from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
from io import open
from builtins import int
from future import standard_library
standard_library.install_aliases()
import sys
import warnings

warnings.filterwarnings("ignore", message="numpy.ufunc size changed")

from wetb.hawc2.ascii2bin.pandas_dat_ascii2bin import pandas_dat_ascii2bin

class TextUI(object):
    def show_message(self, m):
        print (m)

    def exec_long_task(self, text, allow_cancel, task, *args, **kwargs):
        print (text)
        return task(*args, **kwargs)

sys.path.append(".")

def size_from_file(selfilename):
    with open(selfilename, encoding='utf-8') as f:
        info = f.readlines()[8].split()
        scans = int(info[0])
        no_sensors = int(info[1])
    return (scans, no_sensors)

def ascii2bin(ascii_selfilename, bin_selfilename=None, ui=TextUI()):

    # Convert dat file
    ascii_datfilename = ascii_selfilename.replace(".sel", '.dat')
    if bin_selfilename is None:
        bin_selfilename = ascii_selfilename[:-4] + "_bin.sel"


    # Read, convert and write sel file
    with open(ascii_selfilename, encoding='utf-8') as f:
        lines = f.readlines()

    if "BINARY" in lines[8]:
        ui.show_message("%s is already binary" % ascii_selfilename)
    else:
        #lines[1] = "  Version ID : Pydap %d.%d\n" % (version.__version__[:2])
        lines[5] = "  Result file : %s.dat\n" % bin_selfilename[:-4]
        lines[8] = lines[8].replace("ASCII", 'BINARY')
        lines[-1] = lines[-1].strip() + "\n"
        lines.append("Scale factors:\n")

        scale_factors = pandas_dat_ascii2bin(ascii_datfilename, bin_selfilename.replace('.sel', '.dat'), ui)
        for sf in scale_factors:
            lines.append("  %.5E\n" % sf)
        with open(bin_selfilename, 'w', encoding='utf-8') as f:
            f.writelines(lines)
        if ui is not None:
            ui.show_message("Finish converting %s to %s" % (ascii_selfilename, bin_selfilename))

if __name__ == "__main__":
    if len(sys.argv) < 2:
        print ("syntax: ascii2bin ascii_sel_filename [bin_sel_filename]")
    elif len(sys.argv) == 2:
        ascii2bin(sys.argv[1])
    else:
        ascii2bin(sys.argv[1], sys.argv[2])



'''
Created on 11. apr. 2017

@author: mmpe
'''
import struct
import numpy as np
import os

def load(filename, name_stop=8, dtype=np.float):
    """
    Parameters
    ----------
    - filename
    - name_stop : int or str
        if int: Number of characters for name
        if str: name-description delimiter, e.g. " "
    - dtype    
    """
    if isinstance(filename, str):
        fid = open(filename,'rb')
    elif hasattr(filename, "name"):
        fid = filename
        filename = fid.name
    try:
        _ = struct.unpack('i', fid.read(4))
        _ = struct.unpack('i', fid.read(4))
        title = fid.read(60).strip()
     
        _ = struct.unpack('i', fid.read(4))
        _ = struct.unpack('i', fid.read(4))
        no_sensors = struct.unpack('i', fid.read(4))[0]
     
        sensor_numbers = [struct.unpack('i', fid.read(4))[0] for _ in range(no_sensors)]
        _ = struct.unpack('i', fid.read(4))
        _ = struct.unpack('i', fid.read(4))
        time_start = struct.unpack('f', fid.read(4))[0]
        time_step = struct.unpack('f', fid.read(4))[0]
        scale_factors = np.array([struct.unpack('f', fid.read(4))[0] for _ in range(no_sensors)], dtype=np.double)
        data = np.fromstring(fid.read(), 'int16').astype(dtype)
    finally: 
        fid.close()
 
#     if title.isalnum():
#         self.set_info(title, "", self.filename)
#     else:
#         self.set_info(os.path.basename(self.name), "", self.name)
    try:
        data = data.reshape(len(data) // no_sensors, no_sensors)
    except ValueError:
        raise ValueError("The number of data values (%d) is not divisible by the number of sensors (%d)" % (len(data), no_sensors))
 
    for i in range(data.shape[1]):
        data[:, i] *= scale_factors[i]
    no_scans = data.shape[0]
     
 
    # Load sensor file if exists
    
    
    sensor_filename = os.path.join(os.path.dirname(filename), "sensor")
    sensor_info = {info[0]:info[1:] for info in read_sensor_info(sensor_filename, name_stop) }

    # set gain and offset for "Time"
    gains = []
    offsets = []
    names, units, descriptions = [], [], []
    
    for sensor_nr in sensor_numbers:
 
        name, unit, description, gain, offset = sensor_info.get(sensor_nr, ["Attribute %d"%sensor_nr, '-','',1,0])
        if sensor_nr==1 and name=="Time" and unit=="s":
            data = data[:,1:]
            continue
        names.append(name)
        units.append(unit)
        descriptions.append(description)
        gains.append(gain)
        offsets.append(offset)
    
    time = np.arange(time_start, time_start + data.shape[0] * time_step, time_step).reshape((no_scans, 1))
    #data = np.concatenate((time, data), axis=1)
    #gains = np.r_[1,gains]
    #offsets = np.r_[0,offsets]
    # self[:]*=self.gains
    # self[:]+=self.offsets
    info = {"name": title,
            "description": filename,
            "attribute_names": names, 
            "attribute_units": units, 
            "attribute_descriptions": descriptions}
    return time, data, info 


def read_sensor_info(sensor_file, name_stop=8):
    """
    Parameters
    ----------
    - sensor_file
    - name_stop : int or str
        if int: Number of characters for name
        if str: name-description delimiter, e.g. " "    
    """

    if hasattr(sensor_file, 'readlines'):
        sensor_info_lines = sensor_file.readlines()[2:]
    else:
        with open(sensor_file, encoding="utf-8") as fid:
            sensor_info_lines = fid.readlines()[2:]
    sensor_info = []
    for line in sensor_info_lines:
        # while "  " in line:
        #    line = line.replace("  "," ")
        line = line.strip().split()
        nr = int(line[0])
        gain = float(line[1])
        offset = float(line[2])
        unit = line[5]
        if isinstance(name_stop,int):
            name_desc = " ".join(line[6:])
            name = name_desc[:name_stop].strip()
            description = name_desc[name_stop:].strip()
        elif isinstance(name_stop,str):
            name_desc = (" ".join(line[6:])).split(name_stop)
            name = name_desc[0].strip()
            description = name_stop.join(name_desc[1:]).strip()
            

        sensor_info.append((nr, name, unit, description, gain, offset))
    return sensor_info

# def save(dataset, filename):
#     ds = dataset
#     # Write int data file
#     f = open(filename, 'wb')
#     time_att = ds.basis_attribute()
#     sensors = [s for s in ds.attributes() if s is not time_att]
#     
#     if isinstance(ds, FLEX4Dataset):
#         data = ds[:]  # (ds[:]-ds.offsets)/ds.gains
#     else:
#         data = ds[:]
#     if time_att.index != -1:  # time_att may be "Index" with index=-1 if "Time" not exists
#         data = np.delete(data, time_att.index, axis=1)
#     f.write(struct.pack('ii', 0, 0))  # 2x empty int
#     title = ("%-60s" % str(ds.name)).encode()
#     f.write(struct.pack('60s', title))  # title
#     f.write(struct.pack('ii', 0, 0))  # 2x empty int
#     ns = len(sensors)
#     f.write(struct.pack('i', ns))
#     f.write(struct.pack('i' * ns, *range(1, ns + 1)))  # sensor number
#     f.write(struct.pack('ii', 0, 0))  # 2x empty int
#     time = ds.basis_attribute()
#     f.write(struct.pack('ff', time[0], time[1] - time[0]))  # start time and time step
# 
#     scale_factors = np.max(np.abs(data), 0) / 32000
#     f.write(struct.pack('f' * len(scale_factors), *scale_factors))
#     # avoid dividing by zero
#     not0 = np.where(scale_factors != 0)
#     data[:, not0] /= scale_factors[not0]
#     #flatten and round
#     data = np.round(data.flatten()).astype(np.int16)
#     f.write(struct.pack('h' * len(data), *data.tolist()))
#     f.close()
# 
#     # write sensor file
#     f = open(os.path.join(os.path.dirname(filename), 'sensor'), 'w')
#     f.write("Sensor list for %s\n" % filename)
#     f.write(" No   forst  offset  korr. c  Volt    Unit   Navn    Beskrivelse------------\n")
#     sensorlineformat = "%3s  %.3f   %.3f      1.00     0.00 %7s %-8s %s\n"
# 
#     if isinstance(ds, FLEX4Dataset):
#         gains = np.r_[ds.gains[1:], np.ones(ds.shape[1] - len(ds.gains))]
#         offsets = np.r_[ds.offsets[1:], np.zeros(ds.shape[1] - len(ds.offsets))]
#         sensorlines = [sensorlineformat % ((nr + 1), gain, offset, att.unit[:7], att.name.replace(" ", "_")[:8], att.description[:512]) for nr, att, gain, offset in zip(range(ns), sensors, gains, offsets)]
#     else:
#         sensorlines = [sensorlineformat % ((nr + 1), 1, 0, att.unit[:7], att.name.replace(" ", "_")[:8], att.description[:512]) for nr, att in enumerate(sensors)]
#     f.writelines(sensorlines)
#     f.close()


'''
Created on 30. mar. 2017

@author: mmpe
'''
import os
import unittest
from wetb import gtsdf
from wetb.signal.fix._rotor_position import fix_rotor_position, find_fix_dt
from wetb.signal.filters._differentiation import differentiation


tfp = os.path.join(os.path.dirname(__file__), 'test_files/')
import numpy as np
class TestFix(unittest.TestCase):


    def testRotorPositionFix(self):
        ds = gtsdf.Dataset(tfp+'azi.hdf5')
        sample_frq = 25
         
        #import matplotlib.pyplot as plt
        #print (find_fix_dt(ds.azi, sample_frq, ds.Rot_cor, plt))
        rp_fit = fix_rotor_position(ds.azi, sample_frq, ds.Rot_cor)
                  
        rpm_pos = differentiation(rp_fit)%180 / 360 * sample_frq * 60
        err_sum = np.sum((rpm_pos - ds.Rot_cor)**2)
         
        self.assertLess(err_sum,40)
        if 0:
            import matplotlib.pyplot as plt
            t = ds.Time-ds.Time[0]
            plt.plot(t, differentiation(ds.azi)%180 / 360 * sample_frq * 60, label='fit')
            plt.plot(t, ds.Rot_cor)
            plt.plot(t, differentiation(rp_fit)%180 / 360 * sample_frq * 60, label='fit')
            plt.ylim(10,16)
            plt.show()
    
    def test_find_fix_dt(self):
        ds = gtsdf.Dataset(tfp+'azi.hdf5')
        sample_frq = 25
         
        self.assertEqual(find_fix_dt(ds.azi, sample_frq, ds.Rot_cor), 4)
        
        
        

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testRotorPositionFix']
    unittest.main()


'''
Created on 06/09/2013

@author: Mads M. Pedersen (mmpe@dtu.dk)
'''
from __future__ import division, print_function, absolute_import, unicode_literals
from future import standard_library
standard_library.install_aliases()


if __name__=="__main__":
    from mmpe.build_exe.cx.build_cx_exe import NUMPY
    from mmpe.build_exe.cx import build_cx_exe
    build_cx_exe.build_exe("ascii2bin.py", version="3.0.1", includes=["'pandas'"], modules=['email', NUMPY])



'''
Created on 05/06/2012

@author: Mads
'''
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from builtins import zip
from future import standard_library
standard_library.install_aliases()
import os
from wetb.wind.utils import xyz2uvw
import wetb.gtsdf
from wetb.wind.shear import power_shear, fit_power_shear, fit_power_shear_ref, \
    log_shear, fit_log_shear, stability_term



import unittest
import numpy as np

import matplotlib.pyplot as plt


all = True
class TestShear(unittest.TestCase):

    def setUp(self):
        self.tfp = os.path.join(os.path.dirname(__file__), 'test_files/')  # test file path

        """
        Sensor list of hdf5 files
        0: WSP gl. coo.,Abs_vhor 85
        1: WSP gl. coo.,Vdir_hor 85
        2: WSP gl. coo.,Abs_vhor 53
        3: WSP gl. coo.,Vdir_hor 53
        4: WSP gl. coo.,Abs_vhor 21
        5: WSP gl. coo.,Vdir_hor 21
        6: WSP gl. coo.,Vx 85
        7: WSP gl. coo.,Vy 85
        8: WSP gl. coo.,Vz 85
        9: WSP gl. coo.,Vx 53
        10: WSP gl. coo.,Vy 53
        11: WSP gl. coo.,Vz 53
        12: WSP gl. coo.,Vx 21
        13: WSP gl. coo.,Vy 21
        14: WSP gl. coo.,Vz 21
        """

    def test_power_shear_fit(self):
        z = [30,50,70]
        a,u_ref = .3,10
        u = power_shear(a,50,u_ref)(z)
        np.testing.assert_array_almost_equal(fit_power_shear_ref(zip(z,u), 50), [a,u_ref],3)
        if 0:
            import matplotlib.pyplot as plt
            fit_power_shear_ref(zip(z,u), 50, plt)
            plt.show()
        
    def test_power_shear_fit_nan(self):
        z = [30,50,70]
        a,u_ref = .3,10
        u = power_shear(a,50,u_ref)(z)
        u[2] = np.nan
        np.testing.assert_array_almost_equal(fit_power_shear_ref(zip(z,u), 50), [a,u_ref],3)
        u[:] = np.nan
        np.testing.assert_array_almost_equal(fit_power_shear_ref(zip(z,u), 50), [np.nan, np.nan],3)

    def test_power_shear(self):
        if all:
            _, data, _ = wetb.gtsdf.load(self.tfp + 'wind3.hdf5')
            u20 = data[:, 4]
            u70 = data[:, 6]
            z_u_lst1 = [(70, u70), (20, u20)]
            z_u_lst2 = [ (80, 6.93121004105), (100, 7.09558010101), (60, 6.58919000626), (40, 6.12744998932)]

            alpha = fit_power_shear(z_u_lst2)
            self.assertAlmostEqual(alpha, 0.163432280539)
            if 0:
                z = np.arange(10, 100)
                plt.plot([np.mean(u20), np.mean(u70)], [20, 70], 'bo')
                alpha = fit_power_shear(z_u_lst1)
                plt.plot(power_shear(alpha, 70, z, np.mean(u70)), z, 'b')

                z, u = np.array(z_u_lst2).T
                plt.plot(u, z, 'ro')
                alpha = fit_power_shear(z_u_lst2)
                z = np.sort(z)
                plt.plot(power_shear(alpha, 80, z, 6.9), z, 'r')
                plt.show()




    def test_fit_power_shear1(self):

        time, data, info = wetb.gtsdf.load(self.tfp + 'shear_noturb_85.hdf5')  #shear, alpha = 0.5

        wsp85 = data[:, 0]
        wsp53 = data[:, 2]
        wsp21 = data[:, 4]
        u85 = data[:, 7]
        u21 = data[:, 13]

        self.assertAlmostEqual(fit_power_shear([(85, wsp85), (53, wsp53), (21, wsp21)]), 0.5, delta=0.001)
        self.assertAlmostEqual(fit_power_shear([(85, wsp85), (21, wsp21)]), 0.5, delta=0.001)
        self.assertAlmostEqual(fit_power_shear([(85, u85), (21, u21)]), 0.5, delta=0.001)

    def test_fit_power_shear_ref(self):

        time, data, info = wetb.gtsdf.load(self.tfp + 'shear_noturb.hdf5')  #shear, alpha = 0.5

        wsp85 = data[:, 0]
        wsp53 = data[:, 2]
        wsp21 = data[:, 4]

        alpha, u_ref = fit_power_shear_ref([(85, wsp85), (21, wsp21)], 87.13333)
        self.assertAlmostEqual(alpha, .5, delta=.001)
        self.assertAlmostEqual(u_ref, 9, delta=.01)

        alpha, u_ref = fit_power_shear_ref([(85, wsp85), (53, wsp53), (21, wsp21)], 87.13333)
        self.assertAlmostEqual(alpha, .5, delta=.001)
        self.assertAlmostEqual(u_ref, 9, delta=.01)


    def test_fit_power_shear3(self):

        time, data, info = wetb.gtsdf.load(self.tfp + 'shear.hdf5')  #shear, alpha = 0.5
        #print "\n".join(["%d: %s" % (i, n) for i, n in enumerate(info['attribute_names'])])
        """
        0: WSP gl. coo.,Abs_vhor 85
        1: WSP gl. coo.,Vdir_hor 85
        2: WSP gl. coo.,Abs_vhor 53
        3: WSP gl. coo.,Vdir_hor 53
        4: WSP gl. coo.,Abs_vhor 21
        5: WSP gl. coo.,Vdir_hor 21
        6: WSP gl. coo.,Vx 85
        7: WSP gl. coo.,Vy 85
        8: WSP gl. coo.,Vz 85
        9: WSP gl. coo.,Vx 53
        10: WSP gl. coo.,Vy 53
        11: WSP gl. coo.,Vz 53
        12: WSP gl. coo.,Vx 21
        13: WSP gl. coo.,Vy 21
        14: WSP gl. coo.,Vz 21
        """
        wsp85 = data[:, 0]
        wsp53 = data[:, 2]
        wsp21 = data[:, 4]
        u85 = data[:, 7]
        u21 = data[:, 13]

        alpha, u_ref = fit_power_shear_ref([(85, wsp85), (21, wsp21)], 87.13333)
        self.assertAlmostEqual(alpha, .5, delta=.2)
        self.assertAlmostEqual(u_ref, 9, delta=.01)


    def test_log_shear(self):
        shear = log_shear(2, 3)
        self.assertAlmostEqual(shear(9), 5.49306144)

    def test_fit_log_shear(self):
        zu = [(85, 8.88131), (21, 4.41832)]
        u_star, z0 = fit_log_shear(zu)
        if 0:
            for z, u in zu:
                plt.plot(u, z, 'r.')
            z = np.arange(10, 100)
            plt.plot(log_shear(u_star, z0, z), z)
            plt.show()

        shear = log_shear(u_star, z0)
        for z,u in zu:
            self.assertAlmostEqual(u, shear(z), 4)


    def test_show_log_shear(self):
        if 0:
            for ustar in [1, 2]:
                for z0 in [1, 10]:
                    z = np.arange(z0, 200)
                    plt.plot(log_shear(ustar, z0, z), z, label="z0=%d, u*=%d" % (z0, ustar))
            plt.yscale('log')
            plt.legend()
            plt.show()

    def test_show_log_shear_stability(self):
        if 0:
            z0 = 1
            ustar = 1
            z = np.arange(z0, 200)
            for L in [-2000, -100, 100, 2000]:
                plt.plot(log_shear(ustar, z0, z, L), z, label="L=%d" % (L))
            #yscale('log')
            plt.legend()
            plt.show()



if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



'''
Created on 23. dec. 2016

@author: mmpe
'''
import unittest
from wetb.utils.cluster_tools.ssh_client import SSHClient
import os
from wetb.utils.text_ui import TextUI


try:
    import sys
    import wetb
    sys.path.append(os.path.dirname(wetb.__file__) + "/../../x")
    import x
except:
    x=None

import io
from wetb.utils.timing import print_time
import shutil
import getpass

class sshrisoe_interactive_auth_handler(object):
    def __init__(self, password):
        self.password = password
              
    def __call__(self, title, instructions, prompt_list):
        if prompt_list:
            if prompt_list[0][0]=="AD Password: ":
                return [self.password]
            return [getpass.getpass(prompt_list[0][0])]
        return []
    
tfp = os.path.join(os.path.dirname(__file__), 'test_files/')
all = 1
class TestSSHClient(unittest.TestCase):

    def setUp(self):
        if x:
            self.ssh = SSHClient('gorm', 'mmpe',x.mmpe)


    def test_execute(self):
        if 0 or all:
            if x:
                _,out,_ = self.ssh.execute("ls -a")
                ssh_ls = ";".join(sorted(out.split("\n"))[3:]) #Exclude ['', '.', '..']
                win_ls = ";".join(sorted(os.listdir(r"z:")))
                self.assertEqual(ssh_ls, win_ls)

    def test_file_transfer(self):
        if 0 or all:
            if x:
                self.ssh.execute("rm -f tmp.txt")
                io.StringIO()
                
                txt = "Hello world"*1000000
                f = io.StringIO(txt)
                f.seek(0)
                print ("start upload")
                self.ssh.upload(f, "tmp.txt", callback = TextUI().progress_callback("Uploading"))
                print ("endupload")
                _,out,_ = self.ssh.execute("cat tmp.txt")
                self.assertEqual(out, txt)
                fn = tfp + "tmp.txt"
                if os.path.isfile (fn):
                    os.remove(fn)
                self.assertFalse(os.path.isfile(fn))
                self.ssh.download("tmp.txt", fn)
                with open(fn) as fid:
                    self.assertEqual(fid.read(), txt)
        
    
    def test_folder_transfer(self):
        if 0 or all:
            if x:
                p = r"C:\mmpe\HAWC2\models\version_12.3beta/"
                p = r'C:\mmpe\programming\python\WindEnergyToolbox\wetb\hawc2\tests\test_files\simulation_setup\DTU10MWRef6.0_IOS/'
                self.ssh.execute("rm -r -f ./tmp_test")
                self.ssh.upload_files(p, "./tmp_test", ["input/"])
                shutil.rmtree("./test/input", ignore_errors=True)
                self.ssh.download_files("./tmp_test", tfp, "input/" )
                os.path.isfile(tfp + "/input/data/DTU_10MW_RWT_Blade_st.dat")
                shutil.rmtree("./test/input", ignore_errors=True)
                 
        
    def test_folder_transfer_specific_files_uppercase(self):
        if 1 or all:
            if x:
                p = tfp
                files = [os.path.join(tfp, "TEST.txt")]
                self.ssh.execute("rm -r -f ./tmp_test")
                self.ssh.upload_files(p, "./tmp_test", file_lst=files)
                self.assertFalse(self.ssh.file_exists("./tmp_test/test.txt"))
                self.assertTrue(self.ssh.file_exists("./tmp_test/TEST.txt"))
                
            
            
    def test_folder_transfer_specific_files(self):
        if 1 or all:
            if x:
                p = r"C:\mmpe\HAWC2\models\version_12.3beta/"
                p = r'C:\mmpe\programming\python\WindEnergyToolbox\wetb\hawc2\tests\test_files\simulation_setup\DTU10MWRef6.0_IOS/'
                files = [os.path.join(os.path.relpath(root, p), f) for root,_,files in os.walk(p+"input/") for f in files]
                self.ssh.execute("rm -r -f ./tmp_test")
                self.ssh.upload_files(p, "./tmp_test", file_lst=files[:5])
                self.ssh.download_files("./tmp_test", tfp + "tmp/", file_lst = files[:3])
                self.assertEqual(len(os.listdir(tfp+"tmp/input/data/")),2)
                shutil.rmtree(tfp + "tmp/")
            
    def test_ssh_gorm(self):
        if 0 or all:
            if x:
                ssh = SSHClient('gorm.risoe.dk', 'mmpe', x.mmpe)
                _,out,_ = ssh.execute("hostname")
                self.assertEqual(out.strip(), "g-000.risoe.dk")
                
    def test_ssh_g047(self):
        if 0 or all:
            if x:
                gateway = SSHClient('gorm.risoe.dk', 'mmpe', x.mmpe)
                ssh = SSHClient('g-047', "mmpe", x.mmpe, gateway=gateway)
                self.assertEqual(ssh.execute('hostname')[1].strip(), "g-047")

    def test_ssh_risoe(self):
        if 0 or all:
            if x:
                ssh = SSHClient('ssh.risoe.dk', 'mmpe', interactive_auth_handler = sshrisoe_interactive_auth_handler(x.mmpe))
                _,out,_ = ssh.execute("hostname")
                self.assertEqual(out.strip(), "ssh-03.risoe.dk")

    def test_ssh_risoe_gorm(self):
        if 0 or all:
            if x:
            
                gateway = SSHClient('ssh.risoe.dk', 'mmpe', password="xxx", interactive_auth_handler = sshrisoe_interactive_auth_handler(x.mmpe))
                ssh = SSHClient('10.40.23.49', 'mmpe', x.mmpe, gateway = gateway)
                _,out,_ = ssh.execute("hostname")
                self.assertEqual(out.strip(), "g-000.risoe.dk")


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


# -*- coding: utf-8 -*-
"""
Created on Tue Mar 10 18:47:32 2015

@author: dave
"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from io import open
from future import standard_library
standard_library.install_aliases()

import os
import copy

class PBSScript(object):
    """
    Generate a PBS script that includes commands such as copying model files
    to the node and copying back the results
    """

    template = """
### Standard Output
#PBS -N [jobname]
#PBS -o [path_pbs_o]
### Standard Error
#PBS -e [path_pbs_e]
#PBS -W umask=[umask]
### Maximum wallclock time format HOURS:MINUTES:SECONDS
#PBS -l walltime=[walltime]
#PBS -lnodes=[lnodes]:ppn=[ppn]
### Queue name
#PBS -q [queue]

### #PBS -a [start_time]
### #PBS -W depend=afterany:[job_id]

### Browse to current working dir
echo ""
cd $PBS_O_WORKDIR
echo "current working dir:"
pwd
echo ""

### ===========================================================================
echo "------------------------------------------------------------------------"
echo "PRELUDE"
echo "------------------------------------------------------------------------"

[prelude]

echo ""
echo "------------------------------------------------------------------------"
echo "EXECUTION"
echo "------------------------------------------------------------------------"

[execution]
### wait for jobs to finish
wait

echo ""
echo "------------------------------------------------------------------------"
echo "CODA"
echo "------------------------------------------------------------------------"

[coda]

echo ""
### ===========================================================================
exit
"""

    def __init__(self):

        # PBS configuration
        self.jobname = 'no_name_job'
        # relative paths with respect to PBS working directory
        self.path_pbs_o = 'pbs_out/dummy.out'
        self.path_pbs_e = 'pbs_out/dummy.err'
        self.path_pbs_i = 'pbs_in/dummy.pbs'
        # absolute path of the PBS working directory
        self.pbsworkdir = './'
        self.umask = '003'
        self.walltime = '00:59:59'
        self.queue = 'workq'
        self.lnodes = '1'
        self.ppn = '1'

        # regarding the job
        # source2node = [ [/abs/src/base/a/b/c, a/b/c.mod] ]
        self.source2node = [] # copy from the source to the node
        # node2source = [ [a/b/d, /abs/src/base/a/b/d.mod] ]
        self.node2source = [] # what to copy back from the node
        self.ziparchives = []
        self.prelude = ''
        self.execution = ''
        self.coda = ''
        self.scratchdir = '/scratch/$USER/$PBS_JOBID/'

    def check_dirs(self):
        """Create the directories of std out, std error and pbs file if they
        do not exist"""
        dnames = set([os.path.dirname(self.path_pbs_o),
                      os.path.dirname(self.path_pbs_e),
                      os.path.dirname(self.path_pbs_i)])
        for dname in dnames:
            if not os.path.exists(os.path.join(self.pbsworkdir, dname)):
                os.makedirs(os.path.join(self.pbsworkdir, dname))

    def create(self, **kwargs):
        """
        path_pbs_e, path_pbs_o, and path_pbs are relative with respect to
        the working dir

        Parameters
        ----------

        template : str, default=PBSSCript.template

        """

        pbs = kwargs.get('template', copy.copy(self.template))
        jobname = kwargs.get('jobname', self.jobname)
        path_pbs_o = kwargs.get('path_pbs_o', self.path_pbs_o)
        path_pbs_e = kwargs.get('path_pbs_e', self.path_pbs_e)
        path_pbs_i = kwargs.get('path_pbs_i', self.path_pbs_i)
        pbsworkdir = kwargs.get('pbsworkdir', self.pbsworkdir)
        umask = kwargs.get('umask', self.umask)
        walltime = kwargs.get('walltime', self.walltime)
        queue = kwargs.get('queue', self.queue)
        lnodes = kwargs.get('lnodes', self.lnodes)
        ppn = kwargs.get('ppn', self.ppn)
#        source2node = kwargs.get('source2node', self.source2node)
#        node2source = kwargs.get('node2source', self.node2source)
#        ziparchives = kwargs.get('ziparchives', self.ziparchives)
        prelude = kwargs.get('prelude', self.prelude)
        execution = kwargs.get('execution', self.execution)
        coda = kwargs.get('coda', self.coda)
        check_dirs = kwargs.get('check_dirs', False)

        if not os.path.isabs(path_pbs_o):
            path_pbs_o = './' + path_pbs_o
        if not os.path.isabs(path_pbs_e):
            path_pbs_e = './' + path_pbs_e

        pbs = pbs.replace('[jobname]', jobname)
        pbs = pbs.replace('[path_pbs_o]', path_pbs_o)
        pbs = pbs.replace('[path_pbs_e]', path_pbs_e)
        pbs = pbs.replace('[umask]', umask)
        pbs = pbs.replace('[walltime]', walltime)
        pbs = pbs.replace('[queue]', queue)
        pbs = pbs.replace('[lnodes]', lnodes)
        pbs = pbs.replace('[ppn]', ppn)

        pbs = pbs.replace('[prelude]', prelude)
        pbs = pbs.replace('[execution]', execution)
        pbs = pbs.replace('[coda]', coda)

        if check_dirs:
            self.check_dirs()

        # write the pbs_script
        with open(os.path.join(pbsworkdir, path_pbs_i), 'w') as f:
            f.write(pbs)


if __name__ == '__main__':
    pass



# -*- coding: utf-8 -*-
"""
Create a master Excel sheet from a series of text files

Command-line usage
------------------
$ python write_master.py [OPTIONS]

Command line options
--------------------
 --folder (default='DLCs')
    folder containing text files
 --filename (default='DLCs.xlsx')
    name of Excel file to save incl. extension
 --fileend (default='.txt')
    text file extention
 --delimiter (default='\t')
    character separating columns in text files

Author: Jenni Rinker, rink@dtu.dk
"""
from __future__ import print_function
from __future__ import division
from __future__ import unicode_literals
from __future__ import absolute_import

from   argparse import ArgumentParser
import numpy as np
import os
import pandas as pd


def write_master(path_to_texts,
                 excel_name='DLCs.xlsx', file_end='.txt',
                 delimiter='\t'):
    """ Write a master Excel sheet from a series of text files

    Parameters
    ----------

    path_to_texts : str
        path to directory with text files

    excel_name : str
        filename of generated master Excel file

    file_end : str
        file ending of text files

    delimiter : str
        column delimiter in text files
    """

    # formatting for header cells
    header_dict = {'bold': True, 'font_color': '#1F497D',
                   'bottom': 2, 'bottom_color': '#95B3D7'}

    # get list of text files
    text_files = [f for f in os.listdir(path_to_texts) \
                                  if f.endswith(file_end)]

    # check if main text file in the specified directory
    if 'Main'+file_end not in text_files:
        raise ValueError('\"Main\" file not in CSV directory')

    # rearrange text files so main page is first and everything
    #   else is alphabetical
    text_files.remove('Main'+file_end)
    text_files = ['Main'+file_end] + sorted(text_files)

    # open excel file
    writer = pd.ExcelWriter(excel_name, engine='xlsxwriter')

    # create workbook and add formast
    workbook  = writer.book
    header    = workbook.add_format(header_dict)

    # loop through text files
    for text_name in text_files:

        # define path to csv file
        text_path = os.path.join(path_to_texts,text_name)

        # read data, write to Excel file, and define worksheet handle
        text_df = pd.read_table(text_path,
                                delimiter=delimiter, dtype=str,
                                header=None)
        text_df.to_excel(writer, sheet_name=text_name.rstrip(file_end),
                         index=False, header=False)
        worksheet = writer.sheets[text_name.rstrip(file_end)]

        # get column widths by calculating max string lenths
        col_widths = text_df.apply(lambda x: np.max([len(str(s)) for s in x]))

        # add formatting
        for i_col, width in enumerate(col_widths):
            worksheet.set_column(i_col, i_col, width)
        if 'Main' in text_name:  # hardcode first column on main tab
            worksheet.set_column('A:A',  16.56)
            for i_row in [6,9,13,18,24,28]:
                worksheet.set_row(i_row, cell_format=header)
        else:
            worksheet.set_row(1, cell_format=header)
        worksheet.set_zoom(zoom=85)

    # save worksheet
    writer.save()


if __name__ == '__main__':

    # define argument parser
    parser = ArgumentParser(description="generator of master excel sheet")
    parser.add_argument('--folder', type=str, default='DLCs', action='store',
                        dest='folder', help='Destination folder name')
    parser.add_argument('--filename', type=str, default='DLCs.xlsx',
                        action='store',  dest='filename',
                        help='Master spreadsheet file name')
    parser.add_argument('--fileend', type=str, default='.txt',
                        action='store',  dest='fileend',
                        help='File extension for fileend files')
    parser.add_argument('--delimiter', type=str, default='\t',
                        action='store',  dest='delimiter',
                        help='Text delimiter in files')
    opt = parser.parse_args()

    # write master Excel files
    write_master(opt.folder,
                 excel_name=opt.filename, file_end=opt.fileend,
                 delimiter=opt.delimiter)



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()
d = None
d = dir()

from .htc_file import HTCFile
from .log_file import LogFile
from .ae_file import AEFile
from .at_time_file import AtTimeFile
from .pc_file import PCFile
from . import shear_file
from .st_file import StFile 

__all__ = sorted([m for m in set(dir()) - set(d)])



'''
Created on 18/11/2015

@author: MMPE
'''
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from io import open
from future import standard_library
standard_library.install_aliases()
import unittest
from wetb.hawc2.log_file import LogFile, \
    INITIALIZATION, SIMULATING, DONE, PENDING, UNKNOWN
import time
from wetb.hawc2 import log_file
import threading
import os
from wetb.hawc2.htc_file import HTCFile

def simulate(file, wait):
    with open(file, 'r') as fin:
        lines = fin.readlines()
    file = file + "_"
    with open(file, 'w'):
        pass
    time.sleep(.1)
    for l in lines:
        with open(file, 'a+') as fout:
            fout.write(l)
        if "Turbulence generation starts" in l or "Log file output" in l:
            time.sleep(0.2)
        time.sleep(wait)

class TestLogFile(unittest.TestCase):

    def setUp(self):
        unittest.TestCase.setUp(self)
        self.tfp = os.path.join(os.path.dirname(__file__), 'test_files/')  # test file path

    def test_from_htcfile(self):
        htcfile = HTCFile(self.tfp + 'logfiles/model/htc/dlc14_iec61400-1ed3/dlc14_wsp10_wdir000_s0000.htc',"../")
        logfile = LogFile.from_htcfile(htcfile, self.tfp + 'logfiles/model/')
        self.assertEqual(logfile.status, DONE)

    def test_missing_logfile(self):
        f = self.tfp + 'logfiles/missing.log'
        logfile = LogFile(f, 200)
        logfile.update_status()
        self.assertEqual(logfile.pct, 0)
        self.assertEqual(logfile.status, log_file.MISSING)


#    def test_is_file_open(self):
#        f = self.tfp + 'logfiles/test.log'
#        with open(f, 'a+'):
#            self.assertTrue(is_file_open(f))
#        with open(f, 'r'):
#            self.assertTrue(is_file_open(f))
#        self.assertFalse(is_file_open(f))

    def test_simulation_init_error(self):
        f = self.tfp + 'logfiles/init_error.log'
        logfile = LogFile(f, 2)
        logfile.update_status()
        self.assertEqual(logfile.pct, 100)
        self.assertEqual(logfile.status, DONE)
        self.assertEqual(logfile.errors, ['*** ERROR *** No line termination in command line            8'])

    def test_init(self):
        f = self.tfp + 'logfiles/init.log'
        logfile = LogFile(f, 200)
        logfile.update_status()
        self.assertEqual(logfile.pct, 0)
        self.assertEqual(logfile.status, INITIALIZATION)
        self.assertEqual(logfile.errors, [])

    def test_turbulence_generation(self):
        f = self.tfp + 'logfiles/turbulence_generation.log'
        logfile = LogFile(f, 200)
        logfile.update_status()
        self.assertEqual(logfile.pct, 0)
        self.assertEqual(logfile.status, INITIALIZATION)
        self.assertEqual(logfile.errors, [])
        self.assertEqual(logfile.lastline, "Turbulence generation starts ...")

    def test_simulation(self):
        f = self.tfp + 'logfiles/simulating.log'
        logfile = LogFile(f, 2)
        logfile.update_status()
        self.assertEqual(logfile.pct, 25)
        self.assertEqual(logfile.status, SIMULATING)
        self.assertEqual(logfile.errors, [])



    def test_finish(self):
        f = self.tfp + 'logfiles/finish.log'
        logfile = LogFile(f, 200)
        self.assertEqual(logfile.pct, 100)
        self.assertEqual(logfile.status, DONE)
        self.assertEqual(logfile.errors, [])
        self.assertEqual(logfile.elapsed_time, 0.8062344)

    def test_HAWC2Version(self):
        f = self.tfp + 'logfiles/finish.log'
        logfile = LogFile(f, 200)
        self.assertEqual(logfile.hawc2version, "HAWC2MB 11.8")


    def test_simulation_error(self):
        f = self.tfp + 'logfiles/simulation_error.log'
        logfile = LogFile(f, 2)
        self.assertEqual(logfile.pct, 100)
        self.assertEqual(logfile.status, DONE)
        self.assertEqual(logfile.errors, ['*** ERROR *** Error opening out .dat file'])

    def test_simulation_error2(self):
        f = self.tfp + 'logfiles/simulation_error2.log'
        logfile = LogFile(f, 2)
        self.assertEqual(logfile.pct, 100)
        self.assertEqual(logfile.status, DONE)
        self.assertEqual(logfile.errors[0], '*** ERROR *** Out of limits in user defined shear field - limit value used')
        self.assertEqual(logfile.error_str(), '30 x *** ERROR *** Out of limits in user defined shear field - limit value used')



    def check(self, logfilename, phases, end_status, end_errors=[]):
        logfile = LogFile(logfilename + "_", 2)
        logfile.clear()
        if os.path.isfile(logfile.filename):
            os.remove(logfile.filename)
        t = threading.Thread(target=simulate, args=(logfilename, 0.0001))
        t.start()
        last_status = None
        last_pct = 0
        while logfile.pct >= 0 and logfile.status != DONE:
            logfile.update_status()
            if logfile.status != last_status or logfile.pct != last_pct:
                last_status = logfile.status
                last_pct = logfile.pct
                if logfile.status in phases:
                    phases.remove(logfile.status)
            time.sleep(0.01)
        self.assertEqual(logfile.pct, 100)
        self.assertEqual(logfile.status, end_status)
        self.assertEqual(logfile.errors, end_errors)
        self.assertFalse(phases)
        t.join()
        os.remove(logfile.filename)


    def test_realtime_test(self):
        self.check(self.tfp + 'logfiles/finish.log',
                   phases=[PENDING, INITIALIZATION, SIMULATING, DONE],
                   end_status=DONE)

    def test_realtime_test2(self):
        self.check(self.tfp + 'logfiles/init_error.log',
           phases=[PENDING, INITIALIZATION, SIMULATING, DONE],
           end_status=DONE,
           end_errors=['*** ERROR *** No line termination in command line            8'])

    def test_realtime_test_simulation_error(self):
        self.check(self.tfp + 'logfiles/simulation_error.log',
                   [PENDING, INITIALIZATION, SIMULATING, DONE],
                   DONE, ['*** ERROR *** Error opening out .dat file'])

    def test_realtime_test_turbulence(self):
        self.check(self.tfp + 'logfiles/finish_turbulencegeneration.log',
                   phases=[PENDING, INITIALIZATION, SIMULATING, DONE],
                   end_status=DONE,
                   end_errors=[])


    def test_remaining(self):
        logfilename = self.tfp + 'logfiles/finish.log'
        logfile = LogFile(logfilename + "_", 2)
        logfile.clear()
        if os.path.isfile(logfile.filename):
            os.remove(logfile.filename)
        t = threading.Thread(target=simulate, args=(logfilename, 0.01))
        t.start()
        last_status = None
        last_pct = 0
        estimated_simulation_time = None
        while logfile.pct >= 0 and logfile.status != DONE:
            if estimated_simulation_time is None and logfile.remaining_time is not None:
                estimated_endtime = time.time() + logfile.remaining_time
            #print (logfile.pct, logfile.remaining_time, logfile.lastline)
            logfile.update_status()
            if logfile.status != last_status or logfile.pct != last_pct:
                last_status = logfile.status
                last_pct = logfile.pct
            time.sleep(0.1)
        t.join()
        self.assertLess(abs(time.time() - estimated_endtime), 0.15)
        os.remove(logfile.filename)


    def test_remaining_str(self):
        logfile = LogFile("f", 2)
        logfile.remaining_time = 5
        self.assertEqual(logfile.remaining_time_str(), "00:05")
        logfile.remaining_time = 60 + 5
        self.assertEqual(logfile.remaining_time_str(), "01:05")
        logfile.remaining_time = 3600 + 120 + 5
        self.assertEqual(logfile.remaining_time_str(), "1:02:05")


    def test_reset(self):
        htcfile = HTCFile(self.tfp + 'logfiles/model/htc/dlc14_iec61400-1ed3/dlc14_wsp10_wdir000_s0000.htc',"../")
        logfile = LogFile.from_htcfile(htcfile, self.tfp + 'logfiles/model/')
        self.assertEqual(logfile.status, DONE)
        logfile.reset()
        self.assertEqual(logfile.status, UNKNOWN)
        self.assertEqual(logfile.txt, "")
        logfile.update_status()
        self.assertEqual(logfile.status, DONE)


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.test_logfile']
    unittest.main()
